#!/bin/sh
# ---------------------------------------------------------------------------
# JOnAS: Java(TM) Open Application Server
# Copyright (C) 1999 Bull S.A.
# Contact: jonas-team@objectweb.org
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Philippe Durieux
# Contributor(s): ______________________________________.
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# -----------------------------------------------------------------------
# Check JONAS_ROOT is set.
# -----------------------------------------------------------------------
if [ ! -d $JONAS_ROOT/lib ]
then
    echo "JONAS_ROOT must be set"
    exit 1
fi

. $JONAS_ROOT/bin/unix/setenv

# ---------------------------------------------
# Get args
# ---------------------------------------------
ARGS=
COSNAMING_PORT=14727
COSNAMING_HOST=localhost
while [ "$#" -gt 0 ]
do	case "$1" in
	-cp)
		shift
		CLASSPATH=$CLASSPATH$SPS$1
		;;
	-D*)
		JAVA_OPTS="$JAVA_OPTS $1"
		;;
	*)
		# all other args are passed "as is" to the java program
		ARGS="$ARGS $1"
		;;
	esac
	shift
done

if [ -f $JONAS_LIB/client.jar ]
then
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/client.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/jmx/jmxri.jar
else
    # Clients may use all these
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/j2ee/ejb.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/j2ee/jaas1_0.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/j2ee/jndi.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/j2ee/jta-spec1_0_1.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/j2ee/jts-spec1_0.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/j2ee/providerutil.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/j2ee/jms.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/jmx/jmxri.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/joram/mom.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/joram/joram.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/joram/JCup.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/log/ow_monolog.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/log/log4j.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/connector/ow_tx_api.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/carol/carol.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/jonathan/jonathan.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/jonathan/kilim.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/jonathan/nanoxml-lite-2.2.1.jar

    # should only put here a minimum client jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_LIB/common/ow_jonas.jar
fi

# ---------------------------------------------
# JAR for web services
# ---------------------------------------------
if [ -d $JONAS_ROOT/lib/ws ]
then

    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/common/xml/xercesImpl.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/common/xml/xmlParserAPIs.jar

    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/commons-discovery.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jaxrpc.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/saaj.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/wsdl4j.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/axis/axis.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/axis/axis-ant.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jaxm-1.1.1/jaxm-api.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jaxm-1.1.1/jaxm-runtime.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jaxr-1.0_03/castor-0.9.3.9-xml.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jaxr-1.0_03/fscontext.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jaxr-1.0_03/jaxr-api.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jaxr-1.0_03/jaxr-ri.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jaxr-1.0_03/soap.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jwsdp-shared/jcert.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jwsdp-shared/jnet.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jwsdp-shared/jsse.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/jwsdp-shared/namespace.jar
    CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/lib/ws/commons-logging.jar
fi

# ---------------------------------------------
# CLASSPATH can be upgraded here : 
# i.e. to run JOnAS examples
# ---------------------------------------------
CLASSPATH=$CLASSPATH$SPS$JONAS_ROOT/examples/classes

# ---------------------------------------------
# Set options for CAROL
# ---------------------------------------------
JAVA_OPTS="$JAVA_OPTS \
    -Djavax.rmi.CORBA.PortableRemoteObjectClass=org.objectweb.carol.rmi.multi.MultiPRODelegate \
    -Djava.naming.factory.initial=org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory \
    -Djava.security.auth.login.config==$JONAS_ROOT/conf/jaas.config \
    -Dorg.omg.PortableInterceptor.ORBInitializerClass.org.objectweb.jonas_tm.ots.OTSORBInitializer \
    -Dorg.omg.PortableInterceptor.ORBInitializerClass.org.objectweb.jonas.security.interceptors.iiop.SecurityInitializer "

$JAVA $JAVA_OPTS $ARGS
