/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.ear;

//import java
import java.net.URL;

/**
 * Class representing an Ear structure which is composed of :
 *    - array of URL of ejb-jar files
 *    - array of URL of war files
 *    - array of URL of rar files
 * @author Florent Benoit
 */
public class Ear implements EarMBean {
    
    /**
     * Deployment descriptor of the ear
     */
    private String deploymentDescriptor;

    /**
     * URLs of ejb-jars files
     */
    private URL[] ejbJars = null;
    
    /**
     * Urls of wars files
     */
    private URL[] wars = null;
    
    /**
     * Urls of rars files
     */
    private URL[] rars = null;
    

    /**
     * Url of this EAR
     */
    private URL earUrl = null;
    
    /**
     * Construct an Ear structure with the specified files
     * @param earUrl the url of this ear
     * @param deploymentDescriptor the deployment descriptor of the file
     * @param ejbJars the URLs of ejb-jar files
     * @param wars the URLs of the war files
     * @param rars the URLs of the rar files
     */
    public Ear(URL earUrl, String deploymentDescriptor, URL[] ejbJars, URL[] wars, URL[] rars) {
        this.earUrl = earUrl;
        this.deploymentDescriptor = deploymentDescriptor;
        this.ejbJars = ejbJars;
        this.wars = wars;
        this.rars = rars;
    }
    
    /**
     * Return the url of this Ear
     * @return the url of this Ear
     */
    public URL getEarUrl() {
        return earUrl;
    }

    /**
     * Return the ejb-jar files
     * @return the ejb-jar files
     */
    public URL[] getEjbJars() {
        return ejbJars;
    }
    
    /**
     * Return the war files
     * @return the war files
     */
    public URL[] getWars() {
        return wars;
    }
    
    /**
     * Return the rar files
     * @return the rar files
     */
    public URL[] getRars() {
        return rars;
    }
    
    /**
     * Return the deployment descriptor of the ear
     * @return the deployment descriptor of the ear
     */
    public String getDeploymentDescriptor() {
        return deploymentDescriptor;
    }

}
