/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s):
 * JOnAS 2.4 Sebastien Chassande-Barrioz (sebastien.chassande@inrialpes.fr)
 * JOnAS 3.0 Eric Hardesty (Eric.Hardesty@bull.com)
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.resource;

import java.net.URL;
import javax.naming.Context;
import javax.resource.spi.ConnectionManager;
import org.objectweb.jonas.service.Service;

public interface ResourceService extends Service {
    /**
     * Create a Resource Adapter
     * @param Context ctx that defines the:
     *     1) name of the resource adapter.
     *     2) information to determine if the rar is in an ear
     * 
     *        the configuration information for this resource adapter 
     *        must be found in the xml files in the rar file 
     *        being deployed
     */
    void createResourceAdapter(Context ctx) throws Exception;

    /**
     * Deploy the given rars of an ear file with the specified parent 
     * classloader (ear classloader). (This method is only used for 
     * the ear applications).
     * @param Context ctx that defines the:
     * This context contains the following parameters :<BR>
     *    - urls the list of the urls of the rars to deploy.<BR>
     *    - earRootURL the URL of the ear application file.<BR>
     *    - earClassLoader the ear classLoader of the j2ee app.<BR>
     *    - altDDs the optional URI of deployment descriptor.<BR>
     * @throws ResourceServiceException if an error occurs during the deployment.
     */
    void deployRars(Context ctx) throws ResourceServiceException;

    /**
     * Undeploy the given rars of an ear file with the specified parent 
     * classloader (ear classloader). (This method is only used for 
     * the ear applications).
     * @param urls the list of the urls of the rars to deploy.
     */
    void unDeployRars(URL[] urls);
}


