/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer: Florent BENOIT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.security.realm.factory;

//import java
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.OptionalDataException;
import java.util.Hashtable;

//import javax
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.spi.ObjectFactory;

//import objectweb.util
import org.objectweb.util.monolog.api.Logger;

//import jonas
import org.objectweb.jonas.common.Log;


/**
 * This class provides an implementation of the abstract JResource factory for 
 * managing users
 * @author Florent Benoit
 */
public abstract class JResourceFactory implements ObjectFactory {


    /**
     * The logger used in JOnAS
     */
    protected static Logger logger = Log.getLogger(Log.JONAS_SECURITY_PREFIX);


    /**
     * Creates a object using the location or reference 
     * information specified. 
     * @param obj the possibly null object containing location or reference 
     * information that can be used in creating an object.
     * @param name the name of this object relative to nameCtx, or null if no 
     * name is specified.
     * @param nameCtx the context relative to which the name parameter is 
     * specified, or null if name is relative to the default initial context.
     * @param environment the possibly null environment that is used in 
     * creating the object. 
     * @return a newly created object with the specific 
     * configuration; null if an object cannot be created. 
     * @throws Exception if this object factory encountered an exception 
     * while attempting to create an object, and no other object factories 
     * are to be tried.
     */
    public abstract Object getObjectInstance(Object obj, Name name, Context nameCtx,
                                    Hashtable environment) throws Exception;


    /** Return an object from an array of bytes.
     * Useful for BinaryRefAddr
     * @param bytes an array of bytes
     * @return an object or null if there is an error of if it's empty
     */
    protected Object getArrayFromBytes(byte[] bytes) {
        //Declaration
        ByteArrayInputStream bis = null;
        ObjectInputStream ois = null;
        Object obj = null;

        if (bytes == null) {
            return null;
        }

        bis = new ByteArrayInputStream(bytes);
        try {
            ois = new ObjectInputStream(bis);
            obj =  ois.readObject();

        } catch (ClassNotFoundException cfe) {
            //obj is null
        } catch (OptionalDataException ode) {
            //obj is null
        } catch (IOException ioe) {
            //obj is null
        } finally {
            try {
                bis.close();
                ois.close();
            } catch (Exception e) {
                //Ignore it
            }
        }
        return obj;
    }
}
