/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.server;

import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.rmi.server.RMIClassLoaderSpi;

/**
 * Class <code>RemoteClassLoaderSpi</code> is the CAROL JRMP CLass Loader SPI for 
 * serialization performances. 
 * 
 * @author  Guillaume Riviere (Guillaume.Riviere@inrialpes.fr)
 */
public class RemoteClassLoaderSpi extends RMIClassLoaderSpi {
    
    private RMIClassLoaderSpi defRMISpi;

    public RemoteClassLoaderSpi() {
        try {
            Class cl = Class.forName("java.rmi.server.RMIClassLoader");
            Method meth = cl.getMethod("getDefaultProviderInstance",  new Class [0]);
            this.defRMISpi = (RMIClassLoaderSpi) meth.invoke(cl, new Object[0]);
        } catch (Exception e) {
            //TraceCarol.error("RemoteClassLoaderSpi error", e);
        }
    }

    public Class loadClass(String codebase, 
                           String name, 
                           ClassLoader defaultLoader) throws MalformedURLException, 
                                                             ClassNotFoundException {
        return defRMISpi.loadClass(codebase, name, defaultLoader);
    }
    
    public Class loadProxyClass(String codebase, 
                                String[] interfaces,
                                ClassLoader defaultLoader) throws MalformedURLException,
                                                                  ClassNotFoundException {
        return defRMISpi.loadProxyClass(codebase, interfaces, defaultLoader);
    }
    
    public ClassLoader getClassLoader(String codebase) throws MalformedURLException {
        return defRMISpi.getClassLoader(codebase);
    }
    
    public String getClassAnnotation(Class cl) {
        ClassLoader loader = cl.getClassLoader();
        if (loader instanceof JClassLoader) {
            return null;
        } else {
            return defRMISpi.getClassAnnotation(cl);
        }
    }
}
