/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * 01/06/15 Regis Le Brettevillois - Libelis / JOnAS team - Evidian
 *          Creation.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.service;

import javax.naming.Context;
import javax.management.NotificationBroadcasterSupport;
import javax.management.MBeanNotificationInfo;

import org.objectweb.jonas.management.ReconfigDispatcher;

/**
 * Abstract implementation of a Service
 */
public abstract class AbsServiceImpl extends ReconfigDispatcher implements Service {

    /**
     * service name
     */
    private String name = null;
    /**
     * service status
     */
    private boolean started = false;

    /**
     * Initialize the service
     * @param ctx configuration of the service
     * @throws ServiceException service initialization failed
     */
    public void init(Context ctx) throws ServiceException {
        this.doInit(ctx);
    }

    /**
     * Start the service
     * @throws ServiceException service start-up failed
     */
    public void start() throws ServiceException {
        this.doStart();
        this.started = true;
    }

    /**
     * Stop the service
     * @throws ServiceException service stopping failed
     */
    public void stop() throws ServiceException {
        if (this.isStarted()) {
            this.started = false;
            this.doStop();
        }
    }

    /**
     * Returns true if the service is started, false otherwise
     * @return true if the service is started
     */
    public boolean isStarted() {
        return this.started;
    }

    /**
     * Returns the service's name
     * @return the service name
     */
    public String getName() {
        return this.name;
    }

    /**
     * Set the service's name
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }


    /**
     * Abstract initialization method to be implemented by sub-classes
     * @param ctx configuration of the service
     * @throws ServiceException service initialization failed
     */ 
    protected abstract void doInit(Context ctx) throws ServiceException;
    /**
     * Abstract start-up method to be implemented by sub-classes
     * @throws ServiceException service start-up failed
     */ 
    protected abstract void doStart() throws ServiceException;
    /**
     * Abstract method for service stopping to be implemented by sub-classes
     * @throws ServiceException service stopping failed
     */
    protected abstract void doStop() throws ServiceException;
}
