/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;
import java.util.SortedMap;
import java.util.TreeMap;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jmx.JonasManagementRepr;

// JMX
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.ObjectName;

// monolog imports
import org.objectweb.util.monolog.api.BasicLevel;


/**
 * Get from the MBean server the list of the registered MBeans
 * @author Guillaume Riviere
 * <p>
 * Contributors:
 * <ul>
 * <li>Adriana Danes
 * <li>Miroslav Halas 
 * <li>Michel-Ange Anton
 * </ul>
 */
public class AllMbeansServiceServlet extends JadminServlet
{

   /**
    * This query is used to retrieve MBeans. By default we retrieve all
    * MBeans. Derived class can define it's own pattern.
    */
   protected ObjectName queryObjName = null;

   /**
    * Default template to render the MBeans. Derived class can define it's own template.
    */
   protected String strTemplateName = "ALLMBEANS.vm";

   /**
    *
    *
    *  @param ctx a Velocity Context object to be filled with
    *             data.  Will be used for rendering this
    *             template
    *  @return Template to be used for request
    */
   public Template handleRequest(HttpServletRequest request, HttpServletResponse response
      , Context context)
   {
      // Exception treatement
      String alertException = null;

      // Get Date
      String date = java.text.DateFormat.getInstance().format(new Date());
      context.put("date", date);

      // Get the name of the current JOnAS server
      boolean existsServer = JonasManagementRepr.isRegistered(JonasObjectName.ejbServer());
      context.put("existsServer", new Boolean(existsServer));
      if (existsServer)
      {
         String serverName = (String) JonasManagementRepr.getAttribute(JonasObjectName.ejbServer()
            , "JonasName");
         context.put("serverName", serverName);
      }
      String mbeansError = "";
      ObjectName currentObjectName = null;
      try
      {
         // Getting all ObjectNames:
         // MHALAS: Added queryObjName
         //java.util.Set allObjectNames = ConnectorFactory.getRMIConnector().queryNames(queryObjName, null);
         java.util.Set allObjectNames = JonasManagementRepr.queryNames(queryObjName);
         // Parsing the ObjectNames
         Vector managedObjects = new Vector();
         Iterator i;

         // Sort the names so they appear in some order and they are more readable
         SortedMap smNames = new TreeMap();
         for (i = allObjectNames.iterator(); i.hasNext(); )
         {
            currentObjectName = (ObjectName) i.next();
            smNames.put(currentObjectName.getCanonicalName(), currentObjectName);
         }

         for (i = smNames.values().iterator(); i.hasNext(); )
         {
            currentObjectName = (ObjectName) i.next();
            //MBeanInfo currentMBeanInfo = ConnectorFactory.getRMIConnector().getMBeanInfo(currentObjectName);
            MBeanInfo currentMBeanInfo = JonasManagementRepr.getMBeanInfo(currentObjectName);
            // Getting all MBeanAttributeInfo:
            MBeanAttributeInfo[] currentMBeanAttributeInfo = currentMBeanInfo.getAttributes();
            // Parsing all MBeanAttributeInfo
            Vector infos = new Vector();
            for (int j = 0; j < currentMBeanAttributeInfo.length; j++)
            {
               String type = currentMBeanAttributeInfo[j].getType();
               String name = currentMBeanAttributeInfo[j].getName();
               Object detail = null;
               try
               {
                  detail = JonasManagementRepr.getAttribute(currentObjectName, name);
               }
               catch (Exception e)
               {
                  //Can't get attribute
                  logger.log(BasicLevel.DEBUG
                     , currentObjectName + " " + name + " NOT FOUND value by default !!!!!!!!!!!!!!!!!!!!!!!!"
                     + "\n" + e.getMessage());
               }

               logger.log(BasicLevel.DEBUG
                  , currentObjectName + ": " + name + ", " + type + ", w="
                  + currentMBeanAttributeInfo[j].isWritable() + ", r="
                  + currentMBeanAttributeInfo[j].isReadable() + ", " + detail);
               ManagedAttributeInfo info = new ManagedAttributeInfo(type
                  , currentMBeanAttributeInfo[j].isWritable()
                  , currentMBeanAttributeInfo[j].isReadable(), name, detail
                  , currentMBeanAttributeInfo[j]);
               infos.add(info);
            }

            // Getting all MBeanAttributeInfo:
            MBeanOperationInfo[] currentMBeanOperationInfo = currentMBeanInfo.getOperations();
            // Parsing all MBeanOperationInfo
            Vector vOperations = new Vector();
            for (int j = 0; j < currentMBeanOperationInfo.length; j++)
            {
               vOperations.add(currentMBeanOperationInfo[j]);
            }

            managedObjects.add(new ManagedObject( (String) currentMBeanInfo.getClassName()
               , currentObjectName, infos, vOperations));
         }
         context.put("managedObjects", managedObjects);
      }
      catch (Exception e)
      {
         mbeansError = "Current bean '" + currentObjectName + "' Error while looking for MBeans: "
            + e.toString();
         e.printStackTrace();
      }
      context.put("mbeansError", mbeansError);

      // Return template
      Template template = null;
      try
      {
         // MHALAS: Added strTemplateName
         template = getTemplate(strTemplateName);
      }
      catch (ParseErrorException pee)
      {
         logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
      }
      catch (ResourceNotFoundException rnfe)
      {
         logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
      }
      catch (Exception e)
      {
         logger.log(BasicLevel.ERROR, "Error " + e);
      }

      return template;
   }
}
