/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jadmin.servlet;

//  standard imports
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

// JMX
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;

// servlet imports
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

//  JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Servlet used to display information about a DataSource.
 * servlet-name : DatabaseServlet
 * url-pattern  : /servlet/Database
 * .vm template : DATABASE
 * called with the following parameters :
 * dsName = name of the DataSource
 * @author Philippe Durieux
 * <p>
 * Contributors:
 * <ul>
 * <li>Adriana Danes
 * </ul>
 */
public class DatabaseServlet extends JadminServlet {
   
    /**
     * Add info in the provided context and return the template to be used.
     * @param req servlet request from client
     * @param res servlet reponse
     * @param ctx a Velocity Context object to be filled with
     *            data.  Will be used for rendering this template
     *  @return Template to be used for request
     */
    public Template handleRequest(HttpServletRequest req, HttpServletResponse res, Context ctx) { 
        // Exception treatement
        String alertException = null;

        // Get Date
        String date = java.text.DateFormat.getInstance().format(new Date());
        ctx.put("date", date);

        // Get request parameters
        String dsName = null;// datasource name
        String jdbcConnMaxAge = null;
        String jdbcConnCheckLevel = null;
        String jdbcTestStatement = null;
        String maxConnPool = null;
        String minConnPool = null;
        boolean save = false;
        // Associated MBean name
        ObjectName mbeanName = null;
        try {
            for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
                String paramName = (String)e.nextElement();
                if (paramName.equals("dsName")) {
                    dsName = req.getParameter("dsName");
                    mbeanName = JonasObjectName.datasource(dsName);
                } else if (paramName.equals("JDBCConnMaxAge")) {
                    jdbcConnMaxAge = req.getParameter("JDBCConnMaxAge");
                    try {
                        int val = (new Integer(jdbcConnMaxAge)).intValue();
                        if (val < 0) {
                            jdbcConnMaxAge = null;
                            throw new Exception("NumberFormatException on " + req.getParameter("JDBCConnMaxAge"));
                        }
                    } catch (NumberFormatException fe) {
                        jdbcConnMaxAge = null;
                        throw new Exception("NumberFormatException on " + req.getParameter("JDBCConnMaxAge"));
                    }
                } else if (paramName.equals("JDBCConnCheckLevel")) {
                    jdbcConnCheckLevel = req.getParameter("JDBCConnCheckLevel");
                    try {
                        int val = (new Integer(jdbcConnCheckLevel)).intValue();
                        if (val < 0) {
                            jdbcConnCheckLevel = null;
                            throw new Exception("NumberFormatException on " + req.getParameter("JDBCConnCheckLevel"));
                        }
                    } catch (NumberFormatException fe) {
                        jdbcConnCheckLevel = null;
                        throw new Exception("NumberFormatException on " + req.getParameter("JDBCConnCheckLevel"));
                    }
                } else if (paramName.equals("JDBCTestStatement")) {
                    jdbcTestStatement = req.getParameter("JDBCTestStatement");
                } else if (paramName.equals("JDBCMaxConnPool")) {
                    maxConnPool = req.getParameter("JDBCMaxConnPool");
                    try {
                        int val = (new Integer(maxConnPool)).intValue();
                        if (val < 0) {
                            maxConnPool = null;
                            throw new Exception("NumberFormatException on " + req.getParameter("JDBCMaxConnPool"));
                        }
                    } catch (NumberFormatException fe) {
                        if (maxConnPool.equals("no max boundary")) {
                            maxConnPool = "-1";
                        } else {
                            maxConnPool = null;
                            throw new Exception("NumberFormatException on " + req.getParameter("JDBCMaxConnPool"));
                        }
                    }
                } else if (paramName.equals("JDBCMinConnPool")) {
                    minConnPool = req.getParameter("JDBCMinConnPool");
                    try {
                        int val = (new Integer(minConnPool)).intValue();
                        if (val < 0) {
                            minConnPool = null;
                            throw new Exception("NumberFormatException on " + req.getParameter("JDBCMinConnPool"));
                        }
                    } catch (NumberFormatException fe) {
                        minConnPool = null;
                        throw new Exception("NumberFormatException on " + req.getParameter("JDBCMinConnPool"));
                    }
                } else if (paramName.equals("save")) {
                    save = true;    
                }
            }
            if (mbeanName != null) {
                if (save) {          
                    Object[] param = {};
                    String[] signature = {};
                    JonasManagementRepr.invoke(mbeanName, "saveConfig", param, signature);  
                } else {
                    if (jdbcConnMaxAge != null) {
                        Integer currentJdbcConnMaxAge = (Integer)JonasManagementRepr.getAttribute(mbeanName, "JDBCConnMaxAge");
                        Integer updateJdbcConnMaxAge = new Integer(jdbcConnMaxAge);
                        if (!currentJdbcConnMaxAge.equals(updateJdbcConnMaxAge)) {
                            JonasManagementRepr.setAttribute(mbeanName, "JDBCConnMaxAge", updateJdbcConnMaxAge);
                        }
                    }
                    if (jdbcConnCheckLevel != null) {
                        Integer currentJdbcConnCheckLevel = (Integer)JonasManagementRepr.getAttribute(mbeanName, "JDBCConnCheckLevel");
                        Integer updatedJdbcConnCheckLevel = new Integer(jdbcConnCheckLevel);
                        if (!currentJdbcConnCheckLevel.equals(updatedJdbcConnCheckLevel)) {
                            JonasManagementRepr.setAttribute(mbeanName, "JDBCConnCheckLevel", updatedJdbcConnCheckLevel );
                        }
                    }
                    if (jdbcTestStatement != null) {
                        String currentJdbcTestStatement = (String)JonasManagementRepr.getAttribute(mbeanName, "JDBCTestStatement");
                        if (!currentJdbcTestStatement.equals(jdbcTestStatement)) {
                            JonasManagementRepr.setAttribute(mbeanName, "JDBCTestStatement", jdbcTestStatement);
                        }
                    }
                    if (maxConnPool != null) {
                        Integer currentMaxConnPool = (Integer)JonasManagementRepr.getAttribute(mbeanName, "JDBCMaxConnPool");
                        Integer updatedMaxConnPool = (Integer)new Integer(maxConnPool);
                        if (!currentMaxConnPool.equals(updatedMaxConnPool)) {
                            JonasManagementRepr.setAttribute(mbeanName, "JDBCMaxConnPool", updatedMaxConnPool);
                        }
                    }
                    if (minConnPool != null) {
                        Integer currentMinConnPool = (Integer)JonasManagementRepr.getAttribute(mbeanName, "JDBCMinConnPool");
                        Integer updatedMinConnPool = (Integer)new Integer(minConnPool);
                        if (!currentMinConnPool.equals(updatedMinConnPool)) {
                            JonasManagementRepr.setAttribute(mbeanName, "JDBCMinConnPool", updatedMinConnPool);
                        }
                    }
                }
            }
        } catch (Exception e) {
            alertException = alert(e.getMessage());
            ctx.put("alertException", alertException);
        } 

        ctx.put("dsName", dsName);
        // Get general information about the datasource
        String jndiName = null;
        String datasourceType = null;
        Integer currentNumberOfJDBCConnectionsOpen = null;
        String info_error = "";
        try {
            jndiName = (String)JonasManagementRepr.getAttribute(mbeanName, "DSName");
            datasourceType = (String)JonasManagementRepr.getAttribute(mbeanName, "DataSourceType");
            currentNumberOfJDBCConnectionsOpen = (Integer)JonasManagementRepr.getAttribute(mbeanName, "CurrentNumberOfJDBCConnectionOpen");
            ctx.put("jndiName", jndiName);
            ctx.put("datasourceType", datasourceType);
            ctx.put("currentNumberOfJDBCConnectionsOpen", currentNumberOfJDBCConnectionsOpen);
        } catch (ManagementException e1) {
            info_error = "Error while getting attribute: " + e1.getMessage();
        }
        ctx.put("info_error", info_error);
        // Get dependences
        String dep_error = "";
        try {
            Object[] param = { jndiName };
            String[] signature = { "java.lang.String" };
            java.util.Set allDep = (java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
                                                                             "getDataSourceDependence", 
                                                                             param, 
                                                                             signature);
            Hashtable dependences = new Hashtable();
            if (allDep.isEmpty()) {
                logger.log(BasicLevel.DEBUG, "Try to get dependences. Empty dependences set"); 
            } else {
                for (Iterator i = allDep.iterator(); i.hasNext() ;) {
                    ObjectName current = (ObjectName)i.next();
                    Dependence dep = new Dependence(current);
                    dependences.put(dep.getEjbName(), dep);
                }	
            }
            ctx.put("dependences", dependences);
        } catch (ManagementException e) {
            dep_error = "Error while getting attribute:" + e.getMessage();
        } catch (MalformedObjectNameException me) {
            dep_error = "Incompatible name:" + me.getMessage();
        } 
        ctx.put("dep_error", dep_error);
        // Get configuration information about the datasource
        String datasourceDescription = null;
        String url = null;
        String className = null;
        String userName = null;
        String password = null;
        String datasourceFactory = null;
        String datasourceXADataName = null;
        Integer JDBCConnMaxAge = null;
        Integer JDBCConnCheckLevel = null;
        String JDBCTestStatement = null;
        Integer JDBCMinConnPool = null;
        Integer JDBCMaxConnPool = null;
        String JDBCSleepTimeOut = null;
        String JDBCDeadLockPool = null;
        String JDBCLoopWaitPool = null;
        String config_error = "";
        try {
            datasourceDescription = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "DataSourceDescription");
            ctx.put("datasourceDescription", datasourceDescription);
            url = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "Url");
            ctx.put("url", url);
            className = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "ClassName");
            ctx.put("className", className);
            userName = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "UserName");
            ctx.put("userName", userName);
            password = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "Password");
            ctx.put("password", password);
            datasourceFactory = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "DataSourceFactory");
            ctx.put("datasourceFactory", datasourceFactory);
            datasourceXADataName = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "DataSourceXADataName");
            ctx.put("datasourceXADataName", datasourceXADataName);  
            JDBCConnMaxAge = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCConnMaxAge");
            ctx.put("JDBCConnMaxAge", JDBCConnMaxAge);
            JDBCConnCheckLevel = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCConnCheckLevel");
            ctx.put("JDBCConnCheckLevel", JDBCConnCheckLevel);
            JDBCTestStatement = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCTestStatement");
            ctx.put("JDBCTestStatement", JDBCTestStatement);
            JDBCMinConnPool = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCMinConnPool");
            if (JDBCMinConnPool == null)
                ctx.put("JDBCMinConnPool", "0");
            else
                ctx.put("JDBCMinConnPool", JDBCMinConnPool);
            JDBCMaxConnPool = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCMaxConnPool");
            if (JDBCMaxConnPool == null || JDBCMaxConnPool.intValue() == -1)
                ctx.put("JDBCMaxConnPool", "no max boundary");
            else
                ctx.put("JDBCMaxConnPool", JDBCMaxConnPool);
            JDBCSleepTimeOut = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCSleepTimeOut");
            if (JDBCSleepTimeOut == null)
                ctx.put("JDBCSleepTimeOut", "");
            else
                ctx.put("JDBCSleepTimeOut", JDBCSleepTimeOut);
            JDBCDeadLockPool = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCDeadLockPool");
            if (JDBCDeadLockPool == null)
                ctx.put("JDBCDeadLockPool", "");
            else
                ctx.put("JDBCDeadLockPool", JDBCDeadLockPool);
            JDBCLoopWaitPool = (String)JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), "JDBCLoopWaitPool");
            if (JDBCLoopWaitPool == null)
                ctx.put("JDBCLoopWaitPool", "");
            else
                ctx.put("JDBCLoopWaitPool", JDBCLoopWaitPool);
        } catch (ManagementException e) {
            config_error = "Error while getting attribute:" + e.getMessage();
        } catch (javax.management.MalformedObjectNameException e2) {
            config_error = "Incompatible name:" + e2.getMessage();
        }
        ctx.put("config_error", config_error);

        Template template = null;
        try {
            template = getTemplate("DATABASE.vm"); 
        } catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

        return template;
    }

}
