/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jadmin.servlet;

// general Java imports
import java.util.Date;
import java.util.HashMap;
import java.util.StringTokenizer;

// servlet imports
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

// Velocity imports
import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.apache.velocity.servlet.VelocityServlet;

import org.objectweb.jonas.common.Log;

// monolog
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Servlet inherited by all other Jadmin servlets.
 * @author Philippe Durieux, Adriana Danes (JOnAS team)
 */
public abstract class JadminServlet extends VelocityServlet {
    static Logger logger = null;

    // number of administrated services
    static int nbServices = 12;

    // The services names as used in the Velocity templates
    static String services[] = {"EJBSERVER", "EAR", "CONTAINER", "WEB", "DATABASE", "RESOURCE", "TRANSACTION", "JMS", "SECURITY", "MAIL", "LOG", "ALLMBEANS"};

    // These names are contained in the URL pattern allowing to call the servlet corresponding to the service 
    static String linkNames[] = {"EJBServer", "EarService", "ContainerService", "WebService", "DatabaseService", "ResourceService", "TransactionService", "JMSService", "SecurityService", "MailService", "LogService", "AllMbeans"};

    // These names give the label in the banner for the selected service
    static String labelNames[] = {"Server Monitoring", "Ear Service", "Container Service", "Web Service", "Database Service", "Resource Service", "Transaction Service", "JMS Service", "Security Service", "Mail Service", "Logging System", "All Mbeans"};



    /**
     * Performs initialization of this servlet. 
     * Called by the servlet container on loading.
     * @param config the ServletConfig object passed to the servlet by the servlet container
     * @throws ServletException if servlet initialization failed
     */
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        logger = Log.getLogger("org.objectweb.jonas.webapp.jadmin.servlet");
    }

    /** 
     * Creates a VelocityContext.
     * @param req the request treated by the servlet
     * @param res the constructed response
     * @return the created VelocityContext
     */
    protected Context createContext(HttpServletRequest req, HttpServletResponse res) { 
        HttpSession session = req.getSession();
        // Try to get the initial VelocityContext
        VelocityContext ictx = (VelocityContext) session.getAttribute("initialContext");
        if (ictx == null) {
            logger.log(BasicLevel.DEBUG, "Create initial context");
            // initial context lost, recreate it
            ictx = new VelocityContext();

            HashMap linkNamesTable = new HashMap(nbServices);
            HashMap labelNamesTable = new HashMap(nbServices);
	
            for (int i = 0; i < nbServices; i++) {
                linkNamesTable.put(services[i], linkNames[i]);
                labelNamesTable.put(services[i], labelNames[i]);
            }
	
            String[] theServices = new String[nbServices];
            for (int i = 0; i < nbServices; i++) {
                theServices[i] = services[i];
            }

            ictx.put("linkNames", linkNamesTable);
            ictx.put("labelNames", labelNamesTable);
            ictx.put("services", theServices);
            ictx.put("nbServices", new Integer(nbServices));

            session.setAttribute("initialContext", ictx);

        }
        VelocityContext ctx = new VelocityContext(ictx);
        logger.log(BasicLevel.DEBUG, "Create a new VelocityContext containing the initial one");
        return ctx;
    }

    /**
     * get the current date
     * @param ctx the current VelocityContext
     */
    protected void getDate(Context ctx) {
        // Get current Date and put it in ctx
        ctx.put("date", java.text.DateFormat.getInstance().format(new Date()));
    }

    /**
     * @param msg The error message.
     * @return a string displaying the error msg in tr tags.
     */
    protected String alert(String msg) {
        StringBuffer sb = new StringBuffer("<script>alert(\"");
        if (msg != null) {
            StringTokenizer st = new StringTokenizer(msg, "\n");
            while (st.hasMoreTokens()) {
                sb.append(st.nextToken() + "\\n");
            }
        } else {
            sb.append("unidentified problem\\n");
        }
        sb.append("\")</script>");
        return new String(sb);
    }    
}
