/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;
import javax.management.ObjectName;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Servlet allowing to load a datasource from a file or a an empty form
 *
 * @author Adriana Danes
 */
public class LoadDataSourceServlet extends JadminServlet {
   
    /**
     *
     *
     *  @param context a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @request the currently treated http request
     *  @response the http response 
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 	
	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get request parameters
	Properties p = new Properties();
	String fileName = null;
	boolean fnProvided = false; // file name provided
    ObjectName dbmMBean = null;
	try {
        dbmMBean = JonasObjectName.databaseService();
	    for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
            String paramName = (String)e.nextElement();
            if (paramName.equals("loadFromFile")) {
                fileName = request.getParameter("loadFromFile");
                fnProvided = true;
                // check if the data source is not already loaded
                Object[] param = { fileName };
                String[] signature = { "java.lang.String" };
                Boolean loaded = (Boolean) JonasManagementRepr.invoke(dbmMBean, "isLoadedDataSource", param, signature);
                if (loaded.booleanValue()) {
                    // this datasource is already loaded
                    alertException = alert(fileName + " already loaded. You should unload it first !");
                    context.put("alertException", alertException);
                    context.put("error", "Cannot load an already loaded datasource");
                    break;
                } else {
                    // get the data source configuration from this file
                    Object[] param1 = { fileName };
                    String[] signature1 = { "java.lang.String" };
                    p = (Properties)JonasManagementRepr.invoke(dbmMBean, "getDataSourcePropertiesFile", param1, signature1);
                }
            }
        }
	    if (alertException == null) {
            context.put("datasourceProperties", p);
            if (fnProvided) {
                context.put("datasourceName", fileName);
            } else {
                context.put("datasourceName", "");
            }
            context.put("fnProvided", new Boolean(fnProvided));
            context.put("error", "");
        }
	} catch (ManagementException e) {
	    alertException = alert(e.getMessage());
	    context.put("alertException", alertException);
	    context.put("error", "Could not read DataSource configuration file");
	}

	Template template = null;
	try {
	    template = getTemplate("LOAD_DATASOURCE.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }

}
