/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
// JMX
import javax.management.ObjectName;
import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.BasicLevel;


public class LogServiceServlet extends JadminServlet {

    /**
     *  @param context a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   
    public Template handleRequest( HttpServletRequest request, 
                                   HttpServletResponse response, 
                                   Context context ) { 

        // Exception treatement
        String alertException = null;

        // Get Date
        String date = java.text.DateFormat.getInstance().format(new Date());
        context.put("date", date);

        // Get the Log MBeanName
        ObjectName logMBeanName = null;
        // The code sequence below is a work around necessary untill succeeding
        // to debug the JonasObjectName.allLogServices() method
        // Normally it should be written : ObjectName logMBeanName = JonasObjectName.allLogServices();
        //
        // Get the Services' MBeans
        java.util.Set mbeansSet = JonasManagementRepr.queryNames(JonasObjectName.allServices());
        for (java.util.Iterator i = mbeansSet.iterator() ; i.hasNext() ;) {
            ObjectName on = (javax.management.ObjectName)i.next();
            String name = on.getKeyProperty("name");
            if (name.equals("log")) {
                String fname = on.getKeyProperty("fname");
                logMBeanName = JonasObjectName.logService(fname);
                logger.log(BasicLevel.DEBUG, "OK, got the logMBeanName");
                break;
            }
        }

        String topicToSet = null;
        String levelToSet = null;
        boolean doSave = false;
        // Get request parameters
        try {
            for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
                String paramName = (String)e.nextElement();
                if (paramName.equals("topic")) {
                    topicToSet = request.getParameter("topic");
                } else if (paramName.equals("level")) {
                    levelToSet =  request.getParameter("level");
                } else if (paramName.equals("save")) {
                    doSave = true;
                }
            }
            if (doSave) {
                Object[] param = {};
                String[] signature = {};
                JonasManagementRepr.invoke(logMBeanName, "saveConfig", param, signature);
            } else if ((topicToSet != null) && (levelToSet != null)) {
                Object[] param = new Object[2];
                param[0] = topicToSet ; 
                param[1] = levelToSet ;
                String[] signature = new String[2];
                signature[0] = "java.lang.String" ; 
                signature[1] = "java.lang.String" ;
                //JonasManagementRepr.invoke(JonasObjectName.AllLogServices(), "setTopicLevel", param, signature);
                if (logMBeanName != null)
                    JonasManagementRepr.invoke(logMBeanName, "setTopicLevel", param, signature);
            }
        } catch (ManagementException e) {
            alertException = alert(e.getMessage());
            context.put("alertException", alertException);
        }

        // Get the name of the current JOnAS server
        boolean existsServer = JonasManagementRepr.isRegistered(JonasObjectName.ejbServer());
        context.put("existsServer", new Boolean(existsServer));
        if (existsServer) {
            String serverName = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasName");
            context.put("serverName", serverName);
        }

        // get information about the Log service
        //boolean existsLogService = JonasManagementRepr.isRegistered(JonasObjectName.allLogServices("trace"));
        boolean existsLogService = JonasManagementRepr.isRegistered(logMBeanName);
        
        context.put("existsLogService", new Boolean(existsLogService));
        if (existsLogService) {
            String info_error = "";
            Vector topics = new Vector();
            try {
                //String[] topicNames = (String [])JonasManagementRepr.getAttribute(JonasObjectName.AllLogServices(), "Topics");
                String[] topicNames = (String [])JonasManagementRepr.getAttribute(logMBeanName, "Topics");
                String topic = null;
                String level = null;
                for (int i = 0 ; i < topicNames.length ; i++) {
                    topic = topicNames[i];
                    Object[] param = { topic };
                    String[] signature = { "java.lang.String" };
                    //level = (String) JonasManagementRepr.invoke(JonasObjectName.allLogServices(), "getTopicLevel", param, signature);
                    level = (String) JonasManagementRepr.invoke(logMBeanName, "getTopicLevel", param, signature);
                    if (!topic.equals("root"))
                        topics.add(new TopicLevel(topic, level));
                    else
                        topics.add(0, new TopicLevel(topic, level));
                }
                context.put("topics", topics);
            } catch (ManagementException e) {
                info_error = e.getMessage();
            }
            context.put("info_error", info_error);
        }

        // Return template
        Template template = null;
        try {
            template = getTemplate("LOG.vm"); 
        } catch (ParseErrorException pee) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

        return template;
    }

    public class TopicLevel {
        String topic;
        String level;

        TopicLevel(String topic, String level) {
            this.topic = topic;
            this.level = level;
        }
	
        public String getTopic() {
            return topic;
        }	
        public String getLevel() {
            return level;
        }
    }
}
