/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.ManagementException;

// JMX
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;

// monolog imports
import org.objectweb.util.monolog.api.BasicLevel;

public class MailFactoryServletM extends JadminServlet {

    /**
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
                                   HttpServletResponse response, 
                                   Context context ) { 

        // Exception treatement
        String alertException = null;

        // Get Date
        String date = java.text.DateFormat.getInstance().format(new Date());
        context.put("date", date);

        // Get request parameters
        // The name of the mail factory
        String name = null;
        // Associated MBean name
        ObjectName mbeanName = null;
        // udated jndi name
        String jndiNameNew = null;
        boolean setJndiName = false;
        // Updated authentication properties
        Properties authPropsNew = new Properties();
        boolean setAuth = false;
        // Updated session properties
        Properties props = new Properties();
        boolean setProps = false; 
        // Updated mime props
        Properties mimeProps = new Properties();
        boolean setMimeProps = false;
        // gets true in case of save config requests
        boolean save = false;
        try {
            // This request parameter is always present as it is added as 'hidden' input by the MAIL_FACTORY.vm
            name = request.getParameter("name");
            mbeanName = JonasObjectName.mimeMailFactory(name);
            for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
                String paramName = (String)e.nextElement();
                if (paramName.equals("jndiName")) {
                    // The jndi name in request param
                    jndiNameNew = request.getParameter("jndiName");
                    // The current jndi name
                    String jndiName = (String)JonasManagementRepr.getAttribute(mbeanName, "Name");
                    if (!jndiName.equals(jndiNameNew)) {
                        if (!jndiNameNew.equals(""))
                            setJndiName = true;
                        else {
                            alertException = alert("Can't accept empty name for JNDI name");
                            context.put("alertException", alertException);
                        }
                    }
                } else if (paramName.equals("propstext")) {
                    setProps = true;
                    StringTokenizer st = new StringTokenizer(request.getParameter("propstext"));
                    while (st.hasMoreTokens()) {
                        String token = st.nextToken();
                        int pos1 = token.indexOf('=');
                        if (pos1 != -1) {
                            String propName = token.substring(0, pos1);
                            int pos2 = token.indexOf(';');
                            if (pos2 != -1) {
                                String propValue = token.substring(pos1+1, pos2);
                                try {
                                    props.setProperty(propName, propValue);
                                } catch(NullPointerException ne) {
                                    // Do nothing if one of the arguments is null
                                }
                            }
                        }	
                    }
                } else if (paramName.equals("username")) {
                    String usernameNew = request.getParameter("username");
                    String passwordNew = request.getParameter("password");
                    authPropsNew.setProperty("mail.authentication.username", usernameNew);
                    authPropsNew.setProperty("mail.authentication.password", passwordNew);
                    // Get Authentification properties
                    Properties authProps = (Properties)JonasManagementRepr.getAttribute(mbeanName, "AuthenticationProperties");
                    String username = (String)authProps.getProperty("mail.authentication.username");
                    String password = (String)authProps.getProperty("mail.authentication.password");
                    if (username != null) {
                        if (!username.equals(usernameNew)) {
                            logger.log(BasicLevel.DEBUG, "Mail Factory Servlet " + name + ", change mail.authentication.username from " + username + " to " + usernameNew);
                            setAuth = true;
                        } 
                    } else {
                        // username was not defined before
                        if (usernameNew.equals("")) {
                            logger.log(BasicLevel.DEBUG, "Mail Factory Servlet " + name + ", mail.authentication.username stays unset");
                        } else {
                            logger.log(BasicLevel.DEBUG, "Mail Factory Servlet " + name + ", set mail.authentication.username to " + usernameNew);
                            setAuth = true;
                        }
                    }
                    if (password != null) {
                        if (!password.equals(passwordNew)) {
                            logger.log(BasicLevel.DEBUG, "Mail Factory Servlet " + name + ", change mail.authentication.password from " + password + " to " + passwordNew);
                            setAuth = true;
                        } 
                    } else {
                        // password was not defined before
                        if (passwordNew.equals("")) {
                            logger.log(BasicLevel.DEBUG, "Mail Factory Servlet " + name + ", mail.authentication.password stays unset");
                        } else {
                            logger.log(BasicLevel.DEBUG, "Mail Factory Servlet " + name + ", set mail.authentication.password to " + passwordNew);
                            setAuth = true;
                        }
                    }
                } else if (paramName.equals("to")) {
                    setMimeProps = true;
                    mimeProps.setProperty("mail.to", request.getParameter("to"));
                    mimeProps.setProperty("mail.cc", request.getParameter("cc"));
                    mimeProps.setProperty("mail.bcc", request.getParameter("bcc"));
                    mimeProps.setProperty("mail.subject", request.getParameter("subject"));
                } else if (paramName.equals("save")) {
                    save = true;
                }
            }
            if (save) {
                Object[] param = {};
                String[] signature = {};
                JonasManagementRepr.invoke(mbeanName, "saveConfig", param, signature);
            } else {
                if (setJndiName) {
                    JonasManagementRepr.setAttribute(mbeanName, "Name", jndiNameNew);
                }
                if (setAuth) {
                    JonasManagementRepr.setAttribute(mbeanName, "AuthenticationProperties", authPropsNew);
                }
                if (setProps) {
                    //PropDump.print("Servlet transmitted session props are:", props, logger, BasicLevel.DEBUG);
                    JonasManagementRepr.setAttribute(mbeanName,
                                                     "SessionProperties",
                                                     props);
                }
                if (setMimeProps) {                   
                    // update the mimePart message properties
                    JonasManagementRepr.setAttribute(mbeanName,
                                                     "MimeMessageProperties",
                                                     mimeProps);
                }
            }
        } catch (MalformedObjectNameException e) {
            alertException = alert(e.getMessage());
            context.put("alertException", alertException);
        }
        if (name == null) {
            alertException = alert("No mail factory name provided !");
            context.put("alertException", alertException);
        } else {
            context.put("factoryName", name);
        }


        // Get informations about the mail factory
        String jndiName = null;
        String factoryType = null;// the java class
        String type = "not supported"; // should be one of "Session" or "MimePartDataSource"
        String to = null;
        String cc = null;
        String bcc = null;
        String subject = null;
        String info_error = "";
        try {
            // Get Mail Service properties
            jndiName = (String)JonasManagementRepr.getAttribute(mbeanName, "Name");
            context.put("jndiName", jndiName);
            logger.log(BasicLevel.DEBUG, "Mail Factory Servlet for MimePartDS " + name + ", JNDI name is : " + jndiName);

            factoryType = (String)JonasManagementRepr.getAttribute(mbeanName, "FactoryType");
            context.put("factoryType", factoryType);
            if (factoryType.equals("javax.mail.Session"))
                type = "Session";
            else if (factoryType.equals("javax.mail.internet.MimePartDataSource"))
                type = "MimePartDataSource";
            context.put("type", type);
            logger.log(BasicLevel.DEBUG, "Mail Factory Servlet for MimePartDS " + name + ", type is : " + factoryType);

            // Get Authentification properties
            Properties authProps = (Properties)JonasManagementRepr.getAttribute(mbeanName, "AuthenticationProperties");
            String username = (String)authProps.getProperty("mail.authentication.username");
            String password = (String)authProps.getProperty("mail.authentication.password");
            if (username != null)
                context.put("username", username);
            else
                context.put("username", "");
            if (password != null)
                context.put("password", password);
            else
                context.put("password", "");

            // Get Session object properties
            props = (Properties)JonasManagementRepr.getAttribute(mbeanName, "SessionProperties");
            String[] sessionProps = new String[props.size()];
            int i=0;
            for(Enumeration e=props.propertyNames() ; e.hasMoreElements() ; i++) {
                String key = (String)e.nextElement();
                String val = (String)props.getProperty(key);
                sessionProps[i] = new String(key + "=" + val + ";");
            }
            context.put("sessionProps", sessionProps);

            // Get MimeMessage specific properties
            mimeProps = (Properties)JonasManagementRepr.getAttribute(mbeanName, "MimeMessageProperties");
            to = (String)mimeProps.getProperty("mail.to");
            cc = (String)mimeProps.getProperty("mail.cc");
            bcc = (String)mimeProps.getProperty("mail.bcc");
            subject = (String)mimeProps.getProperty("mail.subject");
            if (to != null)
                context.put("to", to);
            else
                context.put("to", "");
            if (cc != null)
                context.put("cc", cc);
            else
                context.put("cc", "");
            if (bcc != null)
                context.put("bcc", bcc);
            else
                context.put("bcc", "");
            if (subject != null)
                context.put("subject", subject);
            else
                context.put("subject", "");

        } catch (ManagementException e) {
            logger.log(BasicLevel.ERROR, e.getMessage());
            info_error = "Error while getting attribute - " + e.getMessage();
        }

        // Get dependences
        String dep_error = "";
        try {
            Object[] param = { jndiName };
            String[] signature = { "java.lang.String" };
            java.util.Set allDep = (java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(),
                                                                             "getMailFactoryDependence",
                                                                             param, 
                                                                             signature);
            Hashtable dependences = new Hashtable();
            if (allDep.isEmpty()) {
                logger.log(BasicLevel.DEBUG, "Try to get dependences of mail factory : " + jndiName + " ??. Empty dependences set !"); 
            } else {
                for (Iterator i = allDep.iterator(); i.hasNext() ;) {
                    ObjectName current = (ObjectName)i.next();
                    Dependence dep = new Dependence(current);
                    dependences.put(dep.getEjbName(), dep);
                }
            }
            context.put("dependences", dependences);
        } catch (ManagementException e) {
            dep_error = "Error while getting attribute:" + e.getMessage();
        } catch (MalformedObjectNameException me) {
            dep_error = "Incompatible name:" + me.getMessage();
        } 
        context.put("dep_error", dep_error);

        // Return template
        Template template = null;
        try {
            template = getTemplate("MAIL_FACTORY.vm"); 
        } catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "MailFactoryServletM : parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe ) {
            logger.log(BasicLevel.ERROR, "MailFactoryServletM : template not found " + rnfe);
        } catch( Exception e ) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

        return template;
    }
}
