/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin;

import javax.management.ObjectName;

/**
 * A set of static classes used to build the names of Catalina MBeans used in JOnAS
 */
public class CatalinaObjectName {
   
   public static ObjectName catalinaServer() {
      try {
         return new ObjectName("Catalina:type=Server");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaServices() {
      try {
         return new ObjectName("Catalina:type=Service,*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaService(String p_ServiceName) {
      try {
         return new ObjectName("Catalina:type=Service,name=" + p_ServiceName);
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaEngines() {
      try {
         return new ObjectName("Catalina:type=Engine,*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaEngine(String p_ServiceName) {
      try {
         return new ObjectName("Catalina:type=Engine,service=" + p_ServiceName);
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaConnectors() {
      try {
         return new ObjectName("Catalina:type=Connector,*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaConnectors(String p_ServiceName) {
      try {
         return new ObjectName("Catalina:type=Connector,service=" + p_ServiceName + ",*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaConnectors(String p_ServiceName, String p_Port) {
      try {
         return new ObjectName("Catalina:type=Connector,service=" + p_ServiceName + ",port="
            + p_Port + ",*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaHosts() {
      try {
         return new ObjectName("Catalina:type=Host,*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaHosts(String p_ServiceName) {
      try {
         return new ObjectName("Catalina:type=Host,service=" + p_ServiceName + ",*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaHost(String p_ServiceName, String p_HostName) {
      try {
         return new ObjectName("Catalina:type=Host,service=" + p_ServiceName + ",host="
            + p_HostName);
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaContexts() {
      try {
         return new ObjectName("Catalina:type=Context,*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaContexts(String p_ServiceName) {
      try {
         return new ObjectName("Catalina:type=Context,service=" + p_ServiceName + ",*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaContexts(String p_ServiceName, String p_Host) {
      try {
         return new ObjectName("Catalina:type=Context,service=" + p_ServiceName + ",host=" + p_Host
            + ",*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaContext(String p_ServiceName, String p_Host, String p_Path) {
      try {
         return new ObjectName("Catalina:type=Context,service=" + p_ServiceName + ",host=" + p_Host
            + "path=" + p_Path + ",*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }

   public static ObjectName catalinaContextsByPath(String p_Path) {
      try {
         return new ObjectName("Catalina:type=Context,path=" + p_Path + ",*");
      }
      catch (javax.management.MalformedObjectNameException e) {
         // this should never occur
         return null;
      }
   }
}
