/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Locale;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.util.MessageResources;

import org.objectweb.jonas.jmx.JonasManagementRepr;

/**
 * Test <code>Action</code> that handles events from the tree control test
 * page.
 *
 */

public class EditTopAction extends Action {
   /**
    * The MessageResources we will be retrieving messages from.
    */
   private MessageResources resources = null;

// --------------------------------------------------------- Public Methods

   /**
    * Process the specified HTTP request, and create the corresponding HTTP
    * response (or forward to another web component that will create it).
    * Return an <code>ActionForward</code> instance describing where and how
    * control should be forwarded, or <code>null</code> if the response has
    * already been completed.
    *
    * @param mapping The ActionMapping used to select this instance
    * @param actionForm The optional ActionForm bean for this request (if any)
    * @param request The HTTP request we are processing
    * @param response The HTTP response we are creating
    *
    * @exception IOException if an input/output error occurs
    * @exception ServletException if a servlet exception occurs
    */
   public ActionForward perform(ActionMapping mapping, ActionForm form, HttpServletRequest request
      , HttpServletResponse response)
      throws IOException, ServletException {
      // Acquire the resources that we need
      HttpSession session = request.getSession();
      Locale locale = (Locale) session.getAttribute(Action.LOCALE_KEY);
      if (resources == null) {
         resources = getServlet().getResources();
      }

      // Test if force to reload all the frames
      ActionErrors oActionErrors = (ActionErrors) request.getAttribute(ERROR_KEY);
      if (oActionErrors != null) {
         request.setAttribute("errorServerSelect", new Boolean(true));
      }
      else {
         String sFirstCall = request.getParameter("firstCall");
         if (sFirstCall == null) {
            request.setAttribute("reloadAll", new Boolean(true));
         }
      }

      // Form used
      TopForm topFm = new TopForm();
      request.setAttribute("topForm", topFm);
      try {
         topFm.setServerSelect(JonasManagementRepr.getCurrentRMIConnectorName());
         //System.out.println("topFm.getServerSelect() = " + topFm.getServerSelect());
         ArrayList al = JonasAdminJmx.getListRemoteJonasServerItem();
         //System.out.println("ArrayList al = " + al);

         /* Add item to test
                   if (al.size() > 0)
                   {
            JonasServerItem oItem = (JonasServerItem)al.get(0);
            JonasServerItem oItemNew = new JonasServerItem();
            oItemNew.setNameServer(oItem.getNameServer()+"_BIS");
            oItemNew.setNameRmiConnector(oItem.getNameRmiConnector()+"_BIS");
            al.add(oItemNew);
            oItemNew = new JonasServerItem();
            oItemNew.setNameServer(oItem.getNameServer()+"_TER");
            oItemNew.setNameRmiConnector(oItem.getNameRmiConnector());
            al.add(oItemNew);
                   }
          */

         request.setAttribute("serverList", al);
      }
      catch (Throwable t) {
         String message = "Problem to access to server JOnAS ! : " + t.getMessage();
         getServlet().log(message, t);
         response.sendError(HttpServletResponse.SC_BAD_REQUEST, message);
         return (null);
      }

      // Forward to the jsp.
      return (mapping.findForward("Top"));
   }

}
