/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin;

import java.io.IOException;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.util.MessageResources;

import org.objectweb.jonas.jmx.JonasManagementRepr;

/**
 *
 *
 */

public abstract class JonasBaseAction extends Action {

// --------------------------------------------------------- Instance Variables

   /**
    * The MessageResources we will be retrieving messages from.
    */
   protected MessageResources m_Resources = null;
   protected HttpSession m_Session = null;
   protected ActionErrors m_Errors = null;

// --------------------------------------------------------- Public Methods

   public abstract ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
      , HttpServletRequest p_Request, HttpServletResponse p_Response)
      throws IOException, ServletException;

   /**
    * Process the specified HTTP request, and create the corresponding HTTP
    * response (or forward to another web component that will create it).
    * Return an <code>ActionForward</code> instance describing where and how
    * control should be forwarded, or <code>null</code> if the response has
    * already been completed.
    *
    * @param mapping The ActionMapping used to select this instance
    * @param actionForm The optional ActionForm bean for this request (if any)
    * @param request The HTTP request we are processing
    * @param response The HTTP response we are creating
    *
    * @exception IOException if an input/output error occurs
    * @exception ServletException if a servlet exception occurs
    */
   public ActionForward perform(ActionMapping p_Mapping, ActionForm p_Form
      , HttpServletRequest p_Request, HttpServletResponse p_Response)
      throws IOException, ServletException {
      // Acquire the resources that we need
      m_Session = p_Request.getSession();
      if (m_Resources == null) {
         m_Resources = getServlet().getResources();
      }
      m_Errors = new ActionErrors();

      return executeAction(p_Mapping, p_Form, p_Request, p_Response);
   }

   protected void addGlobalError(Throwable p_Throwable) {
      String sMessResource;
      String sMessageError = p_Throwable.getMessage();
      if (sMessageError == null) {
         sMessResource = m_Resources.getMessage("error.global.log", p_Throwable.getClass().getName());
         getServlet().log(sMessResource, p_Throwable);
         m_Errors.add("error.global", new ActionError("error.global"
            , p_Throwable.getClass().getName()));
      }
      else {
         sMessResource = m_Resources.getMessage("error.global.message.log"
            , p_Throwable.getClass().getName(), sMessageError);
         getServlet().log(sMessResource, p_Throwable);
         m_Errors.add("error.global", new ActionError("error.global.message"
            , p_Throwable.getClass().getName(), sMessageError));
      }
   }

   protected String getStringAttribute(ObjectName p_ObjectName, String ps_AttrName) {
      String s = (String) JonasManagementRepr.getAttribute(p_ObjectName, ps_AttrName);
      return s;
   }

   protected void setStringAttribute(ObjectName p_ObjectName, String ps_AttrName, String p_Value) {
      JonasManagementRepr.setAttribute(p_ObjectName, ps_AttrName, p_Value);
   }

   protected int getIntegerAttribute(ObjectName p_ObjectName, String ps_AttrName) {
      try {
         Integer o = (Integer) JonasManagementRepr.getAttribute(p_ObjectName, ps_AttrName);
         return o.intValue();
      }
      catch (Exception e) {
         // None
      }
      return 0;
   }

   protected void setIntegerAttribute(ObjectName p_ObjectName, String ps_AttrName, int p_Value) {
      JonasManagementRepr.setAttribute(p_ObjectName, ps_AttrName, new Integer(p_Value));
   }

   protected long getLongAttribute(ObjectName p_ObjectName, String ps_AttrName) {
      try {
         Long o = (Long) JonasManagementRepr.getAttribute(p_ObjectName, ps_AttrName);
         return o.longValue();
      }
      catch (Exception e) {
         // None
      }
      return 0;
   }

   protected void setLongAttribute(ObjectName p_ObjectName, String ps_AttrName, long p_Value) {
      JonasManagementRepr.setAttribute(p_ObjectName, ps_AttrName, new Long(p_Value));
   }

   protected boolean getBooleanAttribute(ObjectName p_ObjectName, String ps_AttrName) {
      try {
         Boolean o = (Boolean) JonasManagementRepr.getAttribute(p_ObjectName, ps_AttrName);
         return o.booleanValue();
      }
      catch (Exception e) {
         // None
      }
      return false;
   }

   protected void setBooleanAttribute(ObjectName p_ObjectName, String ps_AttrName, boolean p_Value) {
      JonasManagementRepr.setAttribute(p_ObjectName, ps_AttrName, new Boolean(p_Value));
   }

}