/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.Action;
import org.apache.struts.util.MessageResources;
import org.apache.webapp.admin.ApplicationServlet;
import org.apache.webapp.admin.TreeBuilder;
import org.apache.webapp.admin.TreeControl;
import org.apache.webapp.admin.TreeControlNode;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.service.container.ContainerItem;
import org.objectweb.jonas.webapp.jonasadmin.service.container.CItemByFileComparator;
import org.objectweb.jonas.webapp.jonasadmin.service.ejb.EjbItem;
import org.objectweb.jonas.webapp.jonasadmin.service.ejb.EjbItemByNameComparator;

/**
 * Implementation of <code>TreeBuilder</code> that adds the nodes required
 * for administering the Jonas server.
 *
 */

public class JonasTreeBuilder implements TreeBuilder {
// ----------------------------------------------------- Instance Variables

// ---------------------------------------------------- TreeBuilder Methods

   /**
    * Add the required nodes to the specified <code>treeControl</code>
    * instance.
    *
    * @param treeControl The <code>TreeControl</code> to which we should
    *  add our nodes
    * @param servlet The controller servlet for the admin application
    * @param request The servlet request we are processing
    */
   public void buildTree(TreeControl treeControl, ApplicationServlet servlet
      , HttpServletRequest request) {
      try {
         TreeControlNode rootNode = treeControl.getRoot();
         MessageResources resources = (MessageResources) servlet.getServletContext().getAttribute(
            Action.MESSAGES_KEY);
         getServer(rootNode, resources);
      }
      catch (Throwable t) {
         t.printStackTrace(System.out);
      }
   }

// ------------------------------------------------------ Protected Methods

   /**
    * Append nodes for a defined Jonas server.
    *
    * @param p_RootNode Root node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServer(TreeControlNode p_RootNode, MessageResources p_Resources)
      throws Exception {
      //System.out.println("getServer >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");

      String sServerName = (String) JonasAdminJmx.getJonasServerName();
      //ObjectName objectName = new ObjectName(sServerName);
      String sNodeLabel = p_Resources.getMessage("treenode.jonas.server") + " ( " + sServerName
         + " )";

      //System.out.println("sServerName = " + sServerName);

      TreeControlNode serverNode = new TreeControlNode(sServerName, "icon/BalJonasTree.gif"
         , sNodeLabel
         , "EditJonasServer.do?select=" + URLEncoder.encode(sServerName) + "&nodeLabel="
         + URLEncoder.encode(sNodeLabel), "content", true);
      p_RootNode.addChild(serverNode);

      getMonitoring(serverNode, p_Resources);
      getLogging(serverNode, p_Resources);
      getServices(serverNode, p_Resources);
      getDeployments(serverNode, p_Resources);

      //System.out.println("getServer <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
   }

   /**
    * Append nodes for Monitoring.
    *
    * @param p_ServerNode Server node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *  messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getMonitoring(TreeControlNode p_ServerNode, MessageResources p_Resources)
      throws Exception {
      //System.out.println("getMonitoring >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");

      // Monitoring server Service node
      TreeControlNode node = new TreeControlNode(p_ServerNode.getName() + ".monitoring"
         , "icon/JonasQuestion.gif", p_Resources.getMessage("treenode.jonas.server.monitoring")
         , "EditMonitoring.do", "content", false);
      p_ServerNode.addChild(node);

      //System.out.println("getMonitoring <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
   }

   /**
    * Append nodes for Logging.
    *
    * @param p_ServerNode Server node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getLogging(TreeControlNode p_ServerNode, MessageResources p_Resources)
      throws Exception {
      //System.out.println("getLogging >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");

      // Logging server Service node
      TreeControlNode node = new TreeControlNode(p_ServerNode.getName() + ".logging"
         , "icon/JonasQuestion.gif", p_Resources.getMessage("treenode.jonas.server.logging")
         , "EditLoggingJonas.do", "content", false);
      p_ServerNode.addChild(node);
      //System.out.println("getLogging <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
   }

   /**
    * Append nodes for all defined services for the specified server.
    *
    * @param p_ServerNode Server node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServices(TreeControlNode p_ServerNode, MessageResources p_Resources)
      throws Exception {
      //System.out.println("getServices >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");

      // Services node
      TreeControlNode nodeAllServices = new TreeControlNode(p_ServerNode.getName() + ".services"
         , "icon/JonasQuestion.gif", p_Resources.getMessage("treenode.jonas.server.services"), null
         , "content", false);
      p_ServerNode.addChild(nodeAllServices);

      // Detail Service node
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.earService()) == true) {
         getServiceApp(nodeAllServices, p_Resources);
      }
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.ejbService()) == true) {
         getServiceEjbContainer(nodeAllServices, p_Resources);
      }
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.webContainerService()) == true) {
         getServiceWeb(nodeAllServices, p_Resources);
      }
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.databaseService()) == true) {
         getServiceDatabase(nodeAllServices, p_Resources);
      }
      //if (JonasAdminJmx.hasMBeanName(JonasObjectName.resourceService()) == true)
      //{
      //   getServiceResource(nodeAllServices, p_Resources);
      //}
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.transactionService()) == true) {
         getServiceTransaction(nodeAllServices, p_Resources);
      }
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.jmsService()) == true) {
         getServiceJms(nodeAllServices, p_Resources);
      }
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.mailService()) == true) {
         getServiceMail(nodeAllServices, p_Resources);
      }
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.securityService()) == true) {
         getServiceSecurity(nodeAllServices, p_Resources);
      }
      //System.out.println("getServices <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
   }

   /**
    * Append node for Ear service for the JOnAS services.
    *
    * @param p_NodeServices Services node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServiceApp(TreeControlNode p_NodeServices, MessageResources p_Resources)
      throws Exception {
      // Add Service node
      TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ".ear"
         , "icon/JonasQuestion.gif", p_Resources.getMessage("treenode.jonas.server.services.ear")
         , "ListAppContainers.do", "content", false);
      p_NodeServices.addChild(nodeService);

      // Add App Containers nodes
      getAppContainers(nodeService, p_Resources);
   }

   /**
    * Append node for all App Containers.
    *
    * @param p_NodeParent Server node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getAppContainers(TreeControlNode p_NodeParent, MessageResources p_Resources)
      throws Exception {
      String sPath;
      String sFile;
      String sNodeName;
      ArrayList al = new ArrayList();

      // Get Container List
      Iterator itNames = ((java.util.Set) JonasManagementRepr.getAttribute(JonasObjectName.
         earService(), "EarNames")).iterator();
      while (itNames.hasNext()) {
         sPath = itNames.next().toString();
         sFile = JonasAdminJmx.extractFilename(sPath);
         if (sFile != null) {
            sNodeName = p_NodeParent.getName() + "." + sFile;
            al.add(new ContainerItem(sFile, sPath, sNodeName));
         }
      }
      // Sort list
      Collections.sort(al, new CItemByFileComparator());
      // Build nodes
      ContainerItem oItem;
      for (int i = 0; i < al.size(); i++) {
         oItem = (ContainerItem) al.get(i);
         TreeControlNode node = new TreeControlNode(oItem.getName(), "icon/JonasQuestion.gif"
            , oItem.getFile(), null, "content", false);
         p_NodeParent.addChild(node);
      }
   }

   /**
    * Append node for EJBContainer service for the JOnAS services.
    *
    * @param p_NodeServices Services node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *  messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServiceEjbContainer(TreeControlNode p_NodeServices
      , MessageResources p_Resources)
      throws Exception {
      // Add Service node
      TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ".ejbContainers"
         , "icon/JonasQuestion.gif"
         , p_Resources.getMessage("treenode.jonas.server.services.ejbContainers")
         , "ListContainers.do", "content", false);
      p_NodeServices.addChild(nodeService);
      // Add Containers nodes
      getContainers(nodeService, p_Resources);
   }

   /**
    * Append node for all Containers.
    *
    * @param p_NodeParent Server node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getContainers(TreeControlNode p_NodeParent, MessageResources p_Resources)
      throws Exception {
      String sPath;
      String sFile;
      String sName;
      String sNodeName;
      ObjectName onContainer;
      ArrayList al = new ArrayList();

      // Get Container List
      Iterator itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allContainers()).iterator();
      while (itNames.hasNext()) {
         sPath = JonasAdminJmx.extractValueMbeanName("fname", itNames.next().toString());
         sFile = JonasAdminJmx.extractFilename(sPath);
         if (sFile != null) {
            sNodeName = p_NodeParent.getName() + "." + sFile;
            al.add(new ContainerItem(sFile, sPath, sNodeName));
         }
      }
      // Sort list
      Collections.sort(al, new CItemByFileComparator());
      // Build nodes
      ContainerItem oItem;
      for (int i = 0; i < al.size(); i++) {
         oItem = (ContainerItem) al.get(i);
         TreeControlNode node = new TreeControlNode(oItem.getName(), "icon/JonasQuestion.gif"
            , oItem.getFile()
            , "EditContainer.do?select=" + URLEncoder.encode(oItem.getPath()) + "&node="
            + URLEncoder.encode(oItem.getPath()), "content", false);
         p_NodeParent.addChild(node);
         // Add ejb node
         getEjbs(node, p_Resources, oItem.getPath());
      }
   }

   /**
    * Append node for all Ejbs for a Container.
    *
    * @param p_NodeParent Server node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    * @param p_Filename Filename of container
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getEjbs(TreeControlNode p_NodeParent, MessageResources p_Resources
      , String p_Filename)
      throws Exception {
      String sName;
      Iterator itNames;
      ArrayList al = new ArrayList();

      // get BMP (Entity with Bean-Managed Persistence)
      itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbBmp(p_Filename)).iterator();
      while (itNames.hasNext()) {
         sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
         al.add(new EjbItem(EjbItem.EJB_BMP, sName, p_Filename));
      }

      // get CMP (Entity with Container-Managed Persistence)
      itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbCmp(p_Filename)).iterator();
      while (itNames.hasNext()) {
         sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
         al.add(new EjbItem(EjbItem.EJB_CMP, sName, p_Filename));
      }

      // get SBF (Stateful Session Bean)
      itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbSbf(p_Filename)).iterator();
      while (itNames.hasNext()) {
         sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
         al.add(new EjbItem(EjbItem.EJB_SBF, sName, p_Filename));
      }

      // get SBL (Stateless Session Bean)
      itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbSbl(p_Filename)).iterator();
      while (itNames.hasNext()) {
         sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
         al.add(new EjbItem(EjbItem.EJB_SBL, sName, p_Filename));
      }

      // get MDB (Message Driven Bean)
      itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbMdb(p_Filename)).iterator();
      while (itNames.hasNext()) {
         sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
         al.add(new EjbItem(EjbItem.EJB_MDB, sName, p_Filename));
      }

      // Sort by name
      Collections.sort(al, new EjbItemByNameComparator());

      // Add all node Ejb
      String sNodeName;
      String sAction;
      TreeControlNode nodeEjb;
      EjbItem oEjb;
      for (int i = 0; i < al.size(); i++) {
         oEjb = (EjbItem) al.get(i);
         sNodeName = p_NodeParent.getName() + "." + oEjb.getName();
         sAction = "EditEjb" + oEjb.getTypeString() + ".do?type=" + oEjb.getTypeString() + "&name="
            + URLEncoder.encode(oEjb.getName()) + "&file=" + URLEncoder.encode(p_Filename)
            + "&node=" + URLEncoder.encode(sNodeName);
         // Add node Ejb
         nodeEjb = new TreeControlNode(sNodeName, "icon/JonasQuestion.gif", oEjb.getName(), sAction
            , "content", false);
         p_NodeParent.addChild(nodeEjb);
      }
   }

   /**
    * Append node for Database service for the JOnAS services.
    *
    * @param p_NodeServices Services node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *  messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServiceDatabase(TreeControlNode p_NodeServices, MessageResources p_Resources)
      throws Exception {
      // Add Service node
      TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ".database"
         , "icon/JonasQuestion.gif"
         , p_Resources.getMessage("treenode.jonas.server.services.database"), null, "content", false);
      p_NodeServices.addChild(nodeService);
   }

   /**
    * Append node for Web service for the JOnAS services.
    *
    * @param p_NodeServices Services node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServiceWeb(TreeControlNode p_NodeServices, MessageResources p_Resources)
      throws Exception {
      // Add Service node
      TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ".web"
         , "icon/JonasQuestion.gif", p_Resources.getMessage("treenode.jonas.server.services.web")
         , "ListWebContainers.do", "content", false);
      p_NodeServices.addChild(nodeService);
      // Add Web Containers nodes
      getWebContainers(nodeService, p_Resources);
   }

   /**
    * Append node for all Web Containers.
    *
    * @param p_NodeParent Server node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getWebContainers(TreeControlNode p_NodeParent, MessageResources p_Resources)
      throws Exception {
      String sPath;
      String sFile;
      String sNodeName;
      ArrayList al = new ArrayList();

      // Get Container List
      Iterator itNames = ((java.util.Set) JonasManagementRepr.getAttribute(JonasObjectName.
         webContainerService(), "WarNames")).iterator();
      while (itNames.hasNext()) {
         sPath = itNames.next().toString();
         sFile = JonasAdminJmx.extractFilename(sPath);
         if (sFile != null) {
            sNodeName = p_NodeParent.getName() + "." + sFile;
            al.add(new ContainerItem(sFile, sPath, sNodeName));
         }
      }
      // Sort list
      Collections.sort(al, new CItemByFileComparator());
      // Build nodes
      ContainerItem oItem;
      for (int i = 0; i < al.size(); i++) {
         oItem = (ContainerItem) al.get(i);
         TreeControlNode node = new TreeControlNode(oItem.getName(), "icon/JonasQuestion.gif"
            , oItem.getFile(), null, "content", false);
         p_NodeParent.addChild(node);
      }
   }

   /**
    * Append node for Transaction service for the JOnAS services.
    *
    * @param p_NodeServices Services node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *  messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServiceTransaction(TreeControlNode p_NodeServices
      , MessageResources p_Resources)
      throws Exception {
      // Add Service node
      TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ".transaction"
         , "icon/JonasQuestion.gif"
         , p_Resources.getMessage("treenode.jonas.server.services.transaction"), null, "content", false);
      p_NodeServices.addChild(nodeService);
   }

   /**
    * Append node for Jms service for the JOnAS services.
    *
    * @param p_NodeServices Services node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServiceJms(TreeControlNode p_NodeServices, MessageResources p_Resources)
      throws Exception {
      // Add Service node
      TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ".jms"
         , "icon/JonasQuestion.gif", p_Resources.getMessage("treenode.jonas.server.services.jms"), null
         , "content", false);
      p_NodeServices.addChild(nodeService);
   }

   /**
    * Append node for Mail service for the JOnAS services.
    *
    * @param p_NodeServices Services node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *  messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServiceMail(TreeControlNode p_NodeServices, MessageResources p_Resources)
      throws Exception {
      // Add Service node
      TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ".mail"
         , "icon/JonasQuestion.gif", p_Resources.getMessage("treenode.jonas.server.services.mail"), null
         , "content", false);
      p_NodeServices.addChild(nodeService);
   }

   /**
    * Append node for Resource service for the JOnAS services.
    *
    * @param p_NodeServices Services node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServiceResource(TreeControlNode p_NodeServices, MessageResources p_Resources)
      throws Exception {
      // Add Service node
      TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ".resource"
         , "icon/JonasQuestion.gif"
         , p_Resources.getMessage("treenode.jonas.server.services.resource"), null, "content", false);
      p_NodeServices.addChild(nodeService);
   }

   /**
    * Append node for Security service for the JOnAS services.
    *
    * @param p_NodeServices Services node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *  messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getServiceSecurity(TreeControlNode p_NodeServices, MessageResources p_Resources)
      throws Exception {
      // Add Service node
      TreeControlNode nodeService = new TreeControlNode(p_NodeServices.getName() + ".security"
         , "icon/JonasQuestion.gif"
         , p_Resources.getMessage("treenode.jonas.server.services.security"), null, "content", false);
      p_NodeServices.addChild(nodeService);
   }

   /**
    * Append nodes for Deployments services for the specified server.
    *
    * @param p_ServerNode Server node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    * @exception Exception if an exception occurs building the tree
    */
   protected void getDeployments(TreeControlNode p_ServerNode, MessageResources p_Resources)
      throws Exception {
      // Deployments node
      TreeControlNode nodeDeployments = new TreeControlNode(p_ServerNode.getName() + ".deployments"
         , "icon/JonasQuestion.gif", p_Resources.getMessage("treenode.jonas.server.deployments"), null
         , "content", false);
      p_ServerNode.addChild(nodeDeployments);

      // Detail Service node
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.earService()) == true) {
         getDeploymentEar(nodeDeployments, p_Resources);
      }
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.ejbService()) == true) {
         getDeploymentEjb(nodeDeployments, p_Resources);
      }
      if (JonasAdminJmx.hasMBeanName(JonasObjectName.webContainerService()) == true) {
         getDeploymentWebAppli(nodeDeployments, p_Resources);
      }
      //if (JonasAdminJmx.hasMBeanName(JonasObjectName.resourceService()) == true)
      //{
      //   getDeploymentConnector(nodeDeployments, p_Resources);
      //}
   }

   /**
    * Append node for Ear deployment.
    *
    * @param p_NodeDeployments Deployment node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getDeploymentEar(TreeControlNode p_NodeDeployments, MessageResources p_Resources)
      throws Exception {
      // Add node
      TreeControlNode oNode = new TreeControlNode(p_NodeDeployments.getName() + "."
         + WhereAreYou.DEPLOYMENT_STRING_EAR, "icon/JonasQuestion.gif"
         , p_Resources.getMessage("treenode.jonas.server.deployments."
         + WhereAreYou.DEPLOYMENT_STRING_EAR)
         , "EditDeploy.do?type=" + WhereAreYou.DEPLOYMENT_STRING_EAR, "content", false);
      p_NodeDeployments.addChild(oNode);
   }

   /**
    * Append node for Ejb deployment.
    *
    * @param p_NodeDeployments Deployment node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getDeploymentEjb(TreeControlNode p_NodeDeployments, MessageResources p_Resources)
      throws Exception {
      // Add node
      TreeControlNode oNode = new TreeControlNode(p_NodeDeployments.getName() + "."
         + WhereAreYou.DEPLOYMENT_STRING_JAR, "icon/JonasQuestion.gif"
         , p_Resources.getMessage("treenode.jonas.server.deployments."
         + WhereAreYou.DEPLOYMENT_STRING_JAR)
         , "EditDeploy.do?type=" + WhereAreYou.DEPLOYMENT_STRING_JAR, "content", false);
      p_NodeDeployments.addChild(oNode);

      // Add list files
      //getDeploymentJarFiles(oNode, p_Resources);
   }

   /**
    * Append node for Ejb Files deployment.
    *
    * @param p_NodeParent Deployment node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getDeploymentEjbFiles(TreeControlNode p_NodeParent, MessageResources p_Resources)
      throws Exception {
      String sFile;
      Iterator it = JonasAdminJmx.getJarFilesDeployable().iterator();
      while (it.hasNext()) {
         sFile = it.next().toString();
         // Add node
         TreeControlNode oNode = new TreeControlNode(p_NodeParent.getName() + "." + sFile
            , "icon/JonasQuestion.gif", sFile, null, "content", false);
         p_NodeParent.addChild(oNode);
      }

   }

   /**
    * Append node for Web deployment.
    *
    * @param p_NodeDeployments Deployment node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getDeploymentWebAppli(TreeControlNode p_NodeDeployments
      , MessageResources p_Resources)
      throws Exception {
      // Add node
      TreeControlNode oNode = new TreeControlNode(p_NodeDeployments.getName() + "."
         + WhereAreYou.DEPLOYMENT_STRING_WAR, "icon/JonasQuestion.gif"
         , p_Resources.getMessage("treenode.jonas.server.deployments."
         + WhereAreYou.DEPLOYMENT_STRING_WAR)
         , "EditDeploy.do?type=" + WhereAreYou.DEPLOYMENT_STRING_WAR, "content", false);
      p_NodeDeployments.addChild(oNode);
   }

   /**
    * Append node for Connector deployment.
    *
    * @param p_NodeDeployments Deployment node for the tree control
    * @param p_Resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getDeploymentConnector(TreeControlNode p_NodeDeployments
      , MessageResources p_Resources)
      throws Exception {
      // Add node
      TreeControlNode oNode = new TreeControlNode(p_NodeDeployments.getName() + "."
         + WhereAreYou.DEPLOYMENT_STRING_RAR, "icon/JonasQuestion.gif"
         , p_Resources.getMessage("treenode.jonas.server.deployments."
         + WhereAreYou.DEPLOYMENT_STRING_RAR)
         , "EditDeploy.do?type=" + WhereAreYou.DEPLOYMENT_STRING_RAR, "content", false);
      p_NodeDeployments.addChild(oNode);
   }
}
