/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin;

import java.net.URLEncoder;
import java.util.Iterator;

import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.Action;
import org.apache.struts.util.MessageResources;
import org.apache.webapp.admin.ApplicationServlet;
import org.apache.webapp.admin.TreeBuilder;
import org.apache.webapp.admin.TreeControl;
import org.apache.webapp.admin.TreeControlNode;

import org.objectweb.jonas.jmx.JonasManagementRepr;

/**
 * Implementation of <code>TreeBuilder</code> that adds the nodes required
 * for administering the Jonas server.
 *
 */

public class MBeanTreeBuilder implements TreeBuilder {
// ----------------------------------------------------- Instance Variables

// ---------------------------------------------------- TreeBuilder Methods

   /**
    * Add the required nodes to the specified <code>treeControl</code>
    * instance.
    *
    * @param treeControl The <code>TreeControl</code> to which we should
    *  add our nodes
    * @param servlet The controller servlet for the admin application
    * @param request The servlet request we are processing
    */
   public void buildTree(TreeControl treeControl, ApplicationServlet servlet
      , HttpServletRequest request) {
      try {
         TreeControlNode rootNode = treeControl.getRoot();
         MessageResources resources = (MessageResources) servlet.getServletContext().getAttribute(
            Action.MESSAGES_KEY);
         getMBeans(rootNode, resources);
      }
      catch (Throwable t) {
         t.printStackTrace(System.out);
      }
   }

// ------------------------------------------------------ Protected Methods

   /**
    * Append nodes for all defined MBeans.
    *
    * @param rootNode Root node for the tree control
    * @param resources The MessageResources for our localized messages
    *
    * @exception Exception if an exception occurs building the tree
    */
   protected void getMBeans(TreeControlNode rootNode, MessageResources resources)
      throws Exception {
      //System.out.println("getMBeans >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");

      TreeControlNode nodeMBeans = new TreeControlNode("treenode.allmbeans", "icon/mbeans.gif"
         , resources.getMessage("treenode.allmbeans"), "ListMBeans.do", "content", false);
      rootNode.addChild(nodeMBeans);

      // Detail MBean node
      String sMBeanName;
      String sDomain;
      String sType;
      String sLastDomain = "";
      String sLastType = "";
      ObjectName onMBean;
      TreeControlNode nodeMBean = null;
      TreeControlNode nodeDomain = null;
      TreeControlNode nodeType = null;
      TreeControlNode nodeParent = null;

      Iterator itMBeansNames = JonasAdminJmx.getListMBeanName(null).iterator();
      while (itMBeansNames.hasNext()) {
         sMBeanName = (String) itMBeansNames.next();
         onMBean = new ObjectName(sMBeanName);
         sDomain = onMBean.getDomain();
         sType = onMBean.getKeyProperty("type");

         //System.out.println("sMBeanName = " + sMBeanName);
         //System.out.println("\tsDomain = " + sDomain);
         //System.out.println("\tsType= " + sType);

         nodeParent = nodeMBeans;
         if (sDomain != null) {
            // Domain node
            if (sDomain.equals(sLastDomain) == false) {
               sLastDomain = sDomain;
               nodeDomain = new TreeControlNode("DomainMBean;" + sDomain, "icon/mbeandomain.gif"
                  , sDomain, null, "content", false);
               nodeMBeans.addChild(nodeDomain);
            }
            nodeParent = nodeDomain;

            if (sType != null) {
               // Type node
               if (sType.equals(sLastType) == false) {
                  sLastType = sType;
                  nodeType = new TreeControlNode("TypeMBean;" + sType, "icon/mbeantype.gif", sType, null
                     , "content", false);
                  nodeDomain.addChild(nodeType);
               }
               nodeParent = nodeType;
            }
         }

         // Mbean node
         nodeMBean = new TreeControlNode("MBean;" + sMBeanName, "icon/mbean.gif", sMBeanName
            , "ListMBeanProperties.do?select=" + URLEncoder.encode(sMBeanName) + "&nodeLabel="
            + URLEncoder.encode(sMBeanName), "content", false);
         nodeParent.addChild(nodeMBean);

         // Append MBeanInfo node
         //getMBeanInfo(nodeMBean, onMBean, resources);
      }
      //System.out.println("getMBeans <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
   }

   /**
    * Append nodes Attributes and Operations for a defined MBean.
    *
    * @param nodeMBean The MBean node
    * @param onMBean The MBean Object name
    * @param resources Resource
    * @throws Exception
    */
   protected void getMBeanInfo(TreeControlNode nodeMBean, ObjectName onMBean
      , MessageResources resources)
      throws Exception {
      // Get all infos of a MBean
      MBeanInfo oMBeanInfo = JonasManagementRepr.getMBeanInfo(onMBean);
      // Get attributes infos
      MBeanAttributeInfo[] aoMBeanAttributeInfo = oMBeanInfo.getAttributes();
      if (aoMBeanAttributeInfo.length > 0) {
         // Append attributes node
         TreeControlNode nodeAttributes = new TreeControlNode("Attributes;" + onMBean.toString()
            , "icon/JonasQuestion.gif", resources.getMessage("treenode.allmbeans.attributes"), null
            , "content", false);
         nodeMBean.addChild(nodeAttributes);
         // Loop to append each attribute node
         for (int i = 0; i < aoMBeanAttributeInfo.length; i++) {
            TreeControlNode nodeAttr = new TreeControlNode(String.valueOf(i) + ";"
               + onMBean.toString() + ";" + aoMBeanAttributeInfo[i].getName(), "icon/property.gif"
               , aoMBeanAttributeInfo[i].getName(), null, "content", false);
            nodeAttributes.addChild(nodeAttr);
         }
      }
      // Get operations infos
      MBeanOperationInfo[] aoMBeanOperationInfo = oMBeanInfo.getOperations();
      if (aoMBeanOperationInfo.length > 0) {
         // Append operations node
         TreeControlNode nodeOperations = new TreeControlNode("Operations;" + onMBean.toString()
            , "icon/JonasQuestion.gif", resources.getMessage("treenode.allmbeans.operations"), null
            , "content", false);
         nodeMBean.addChild(nodeOperations);
         // Loop to append each operation node
         for (int i = 0; i < aoMBeanOperationInfo.length; i++) {
            TreeControlNode nodeOpe = new TreeControlNode(String.valueOf(i) + ";"
               + onMBean.toString() + ";" + aoMBeanOperationInfo[i].getName(), "icon/action.gif"
               , aoMBeanOperationInfo[i].getName(), null, "content", false);
            nodeOperations.addChild(nodeOpe);
         }
      }
   }
}
