/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.ActionServlet;
import org.apache.webapp.admin.TreeControl;
import org.apache.webapp.admin.TreeControlNode;



public class WhereAreYou {

// --------------------------------------------------------- Public Constants

   public final static String TAGS_IMAGES_ROOT_KEY = "tagsimagesroot";
   public final static String CONTEXT_MACRO = "${CONTEXT}";

   public final static int DEPLOYMENT_EAR = 1;
   public final static int DEPLOYMENT_JAR = 2;
   public final static int DEPLOYMENT_WAR = 3;
   public final static int DEPLOYMENT_RAR = 4;

   public final static String DEPLOYMENT_STRING_EAR = "ear";
   public final static String DEPLOYMENT_STRING_JAR = "jar";
   public final static String DEPLOYMENT_STRING_WAR = "war";
   public final static String DEPLOYMENT_STRING_RAR = "rar";

// --------------------------------------------------------- Properties variable

   private String imagesRoot = null;
   private TreeControl treeControl = null;
   private String titleContent = null;
   private boolean catalinaEngine = false;
   private boolean catalinaEngineLogger = false;
   private boolean catalinaServer = false;
   private boolean catalinaService = false;
   private String catalinaServiceName = null;
   private boolean myCatalinaContext = false;
   private boolean myCatalinaConnector = false;
   private boolean myJonasAdminServer = false;

   /**
    * Type of deployment in progress : JAR, EAR, WAR, RAR.
    */
   private int currentJonasDeploymentType = 0;

// --------------------------------------------------------- Constructor

   public WhereAreYou() {
      super();
   }

// --------------------------------------------------------- Public Methods

   /**
    * Initialize the properties in only reading.
    *
    * @param p_Servlet The instance of servlet to access to the parameter of the servlet config
    * @param p_Request The request to access to the HTTP informations
    */
   public void initialize(ActionServlet p_Servlet, HttpServletRequest p_Request) {
      // Tags Images Root directory
      String sImagesRoot = (String) p_Servlet.getServletConfig().getInitParameter(
         TAGS_IMAGES_ROOT_KEY);
      if (sImagesRoot != null) {
         sImagesRoot = JonasAdminJmx.replace(sImagesRoot, CONTEXT_MACRO, p_Request.getContextPath());
      }
      this.imagesRoot = sImagesRoot;
      // Initialize Servers properties
      refreshServers(p_Request);

      /*
       java.util.Properties oProps = System.getProperties();
       //oProps.list(System.out);
       String sKey;
       java.util.ArrayList alKey = new java.util.ArrayList(oProps.keySet());
       java.util.Collections.sort(alKey);
       java.util.Iterator it = alKey.iterator();
       System.out.println("------------------------------------------------------------");
       while (it.hasNext())
       {
          sKey = it.next().toString();
          System.out.println(sKey);
          System.out.println(oProps.getProperty(sKey));
          System.out.println("------------------------------------------------------------");
       }
       */

   }

   /**
    * Return the name of the selected node in the tree.
    *
    * @return The name
    */
   public String getSelectedNameNode() {
      if (treeControl != null) {
         TreeControlNode oNode = treeControl.getSelected();
         if (oNode != null) {
            return oNode.getName();
         }
      }
      return null;
   }

   /**
    * Return the selected node in the tree.
    *
    * @return The selected node
    */
   public TreeControlNode getSelectedTreeControlNode() {
      if (treeControl != null) {
         return treeControl.getSelected();
      }
      return null;
   }

   /**
    * Refresh all the servers properties.
    *
    * @param p_Request The request to access to the HTTP informations
    */
   public void refreshServers(HttpServletRequest p_Request) {
      refreshCatalina(p_Request);
      refreshJonas(p_Request);
   }

   /**
    * Refresh the Catalina server properties.
    *
    * @param p_Request The request to access to the HTTP informations
    */
   public void refreshCatalina(HttpServletRequest p_Request) {
      resetCatalina();
      try {
         catalinaServer = JonasAdminJmx.hasMBeanName(CatalinaObjectName.catalinaServer());
         catalinaService = JonasAdminJmx.hasMBeanName(CatalinaObjectName.catalinaServices());
         catalinaServiceName = JonasAdminJmx.extractValueMbeanName("name"
            , JonasAdminJmx.getFirstMBeanName(CatalinaObjectName.catalinaServices()));
         catalinaEngine = JonasAdminJmx.hasMBeanName(CatalinaObjectName.catalinaEngine(
            catalinaServiceName));

         String sPort = String.valueOf(p_Request.getServerPort());
         myCatalinaConnector = JonasAdminJmx.hasMBeanName(CatalinaObjectName.catalinaConnectors(
            catalinaServiceName, sPort));
         myCatalinaContext = JonasAdminJmx.hasMBeanName(CatalinaObjectName.catalinaContextsByPath(
            p_Request.getContextPath()));
         myJonasAdminServer = myCatalinaConnector && myCatalinaContext;

         //catalinaEngineLogger = JonasAdminJmx.hasJmxMBean(sSearch);
      }
      catch (Exception e) {
         // none
      }
   }

   /**
    * Refresh the JOnAS server properties.
    *
    * @param p_Request The request to access to the HTTP informations
    */
   public void refreshJonas(HttpServletRequest p_Request) {
      resetJonas();
      try {
      }
      catch (Exception e) {
         // none
      }
   }

   /**
    * Return in a string the contents of the instance for each propertie.
    * Debug utility.
    *
    * @return The string contents
    */
   public String toString() {
      StringBuffer sb = new StringBuffer();
      sb.append("imagesRoot=[").append(imagesRoot).append("] ");
      sb.append("treeControl=[").append(treeControl).append("] ");
      sb.append("titleContent=[").append(titleContent).append("] ");
      sb.append("catalinaServer=[").append(catalinaServer).append("] ");
      sb.append("catalinaService=[").append(catalinaService).append("] ");
      sb.append("catalinaServiceName=[").append(catalinaServiceName).append("] ");
      sb.append("catalinaEngine=[").append(catalinaEngine).append("] ");
      sb.append("catalinaEngineLogger=[").append(catalinaEngineLogger).append("] ");
      sb.append("myCatalinaConnector=[").append(myCatalinaConnector).append("] ");
      sb.append("myCatalinaContext=[").append(myCatalinaContext).append("] ");
      sb.append("myJonasAdminServer=[").append(myJonasAdminServer).append("] ");
      return sb.toString();
   }

// --------------------------------------------------------- Protected Methods

   /**
    * Reset the Catalina server properties.
    */
   protected void resetCatalina() {
      catalinaEngine = false;
      catalinaEngineLogger = false;
      catalinaServer = false;
      catalinaService = false;
      catalinaServiceName = null;
      myCatalinaContext = false;
      myCatalinaConnector = false;
      myJonasAdminServer = false;
   }

   /**
    * Reset the JOnAS server properties.
    */
   protected void resetJonas() {
   }

// --------------------------------------------------------- Properties Methods

   public String getImagesRoot() {
      return imagesRoot;
   }

   public TreeControl getTreeControl() {
      return treeControl;
   }

   public void setTreeControl(TreeControl treeControl) {
      this.treeControl = treeControl;
   }

   public String getTitleContent() {
      return titleContent;
   }

   public void setTitleContent(String titleContent) {
      this.titleContent = titleContent;
   }

   public boolean isCatalinaEngine() {
      return catalinaEngine;
   }

   public boolean isCatalinaEngineLogger() {
      return catalinaEngineLogger;
   }

   public boolean isCatalinaServer() {
      return catalinaServer;
   }

   public boolean isCatalinaService() {
      return catalinaService;
   }

   public String getCatalinaServiceName() {
      return catalinaServiceName;
   }

   public boolean isMyCatalinaContext() {
      return myCatalinaContext;
   }

   public boolean isMyCatalinaConnector() {
      return myCatalinaConnector;
   }

   public boolean isMyJonasAdminServer() {
      return myJonasAdminServer;
   }

   public int getCurrentJonasDeploymentType() {
      return currentJonasDeploymentType;
   }

   public void setCurrentJonasDeploymentType(int currentJonasDeploymentType) {
      this.currentJonasDeploymentType = currentJonasDeploymentType;
   }

}