/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.deploy;

import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;
import org.objectweb.jonas.webapp.jonasadmin.WhereAreYou;

/**
 *
 */

abstract public class BaseDeployAction extends JonasBaseAction {

// --------------------------------------------------------- Public Methods

   protected int getCurrentJonasDeployment()
      throws Exception {
      int iRet = 0;
      WhereAreYou oWhere = (WhereAreYou) m_Session.getAttribute("WhereAreYou");
      iRet = oWhere.getCurrentJonasDeploymentType();
      return iRet;
   }

   protected void setCurrentJonasDeployment(HttpServletRequest p_Request)
      throws Exception {
      int iDeployment = 0;
      String sDeployment = p_Request.getParameter("type");
      if (sDeployment != null) {
         // Dispatch
         if (sDeployment.equals(WhereAreYou.DEPLOYMENT_STRING_EAR) == true) {
            iDeployment = WhereAreYou.DEPLOYMENT_EAR;
         }
         else if (sDeployment.equals(WhereAreYou.DEPLOYMENT_STRING_JAR) == true) {
            iDeployment = WhereAreYou.DEPLOYMENT_JAR;
         }
         else if (sDeployment.equals(WhereAreYou.DEPLOYMENT_STRING_WAR) == true) {
            iDeployment = WhereAreYou.DEPLOYMENT_WAR;
         }
         else if (sDeployment.equals(WhereAreYou.DEPLOYMENT_STRING_RAR) == true) {
            iDeployment = WhereAreYou.DEPLOYMENT_RAR;
         }
         // Storing
         WhereAreYou oWhere = (WhereAreYou) m_Session.getAttribute("WhereAreYou");
         oWhere.setCurrentJonasDeploymentType(iDeployment);
      }
   }

   /**
    * Return the list of deployable files in using the type of current deployment
    * storing in WhereAreYou instance in session.
    *
    * @return The list of deployable files
    * @throws Exception
    */
   protected ArrayList getListDeployableFiles()
      throws Exception {
      ArrayList al = null;
      switch (getCurrentJonasDeployment()) {
         case WhereAreYou.DEPLOYMENT_EAR:
            al = JonasAdminJmx.getEarFilesDeployable();
            break;
         case WhereAreYou.DEPLOYMENT_JAR:
            al = JonasAdminJmx.getJarFilesDeployable();
            break;
         case WhereAreYou.DEPLOYMENT_WAR:
            al = JonasAdminJmx.getWarFilesDeployable();
            break;
      }
      return al;
   }

   /**
    * Return the list of deployed files in using the type of current deployment
    * storing in WhereAreYou instance in session.
    *
    * @return The list of deployed files
    * @throws Exception
    */
   protected ArrayList getListDeployedFiles()
      throws Exception {
      ArrayList al = null;
      switch (getCurrentJonasDeployment()) {
         case WhereAreYou.DEPLOYMENT_EAR:
            al = JonasAdminJmx.getEarFilesDeployed();
            break;
         case WhereAreYou.DEPLOYMENT_JAR:
            al = JonasAdminJmx.getJarFilesDeployed();
            break;
         case WhereAreYou.DEPLOYMENT_RAR:
            al = JonasAdminJmx.getRarFilesDeployed();
            break;
         case WhereAreYou.DEPLOYMENT_WAR:
            al = JonasAdminJmx.getWarFilesDeployed();
            break;
      }
      return al;
   }

   /**
    * Undeploy a file in using the type of current deployment
    * storing in WhereAreYou instance in session.
    *
    * @param p_Filename Name of file to undeploy
    * @throws Exception
    */
   protected void Undeploy(String p_Filename)
      throws Exception {
      String[] asParam = new String[1];
      asParam[0] = p_Filename;
      String[] asSignature = new String[1];
      asSignature[0] = "java.lang.String";

      switch (getCurrentJonasDeployment()) {
         case WhereAreYou.DEPLOYMENT_EAR:
            JonasManagementRepr.invoke(JonasObjectName.earService(), "unDeployEarMBean", asParam
               , asSignature);
            break;
         case WhereAreYou.DEPLOYMENT_JAR:
            JonasManagementRepr.invoke(JonasObjectName.ejbService(), "removeContainer", asParam
               , asSignature);
            break;
         case WhereAreYou.DEPLOYMENT_RAR:
            System.out.println(" !!!!!!!!!!!!!!!!!!!!! Not implemented !!!!!!!!!!!!!!");
            break;
         case WhereAreYou.DEPLOYMENT_WAR:
            JonasManagementRepr.invoke(JonasObjectName.webContainerService(), "unRegisterWarMBean"
               , asParam, asSignature);
            break;
      }
   }

   /**
    * Deploy a file in using the type of current deployment
    * storing in WhereAreYou instance in session.
    *
    * @param p_Filename Name of file to undeploy
    * @throws Exception
    */
   protected void Deploy(String p_Filename)
      throws Exception {
      String[] asParam = new String[1];
      asParam[0] = p_Filename;
      String[] asSignature = new String[1];
      asSignature[0] = "java.lang.String";

      switch (getCurrentJonasDeployment()) {
         case WhereAreYou.DEPLOYMENT_EAR:
            JonasManagementRepr.invoke(JonasObjectName.earService(), "deployEarMBean", asParam
               , asSignature);
            break;
         case WhereAreYou.DEPLOYMENT_JAR:
            JonasManagementRepr.invoke(JonasObjectName.ejbService(), "createContainer", asParam
               , asSignature);
            break;
         case WhereAreYou.DEPLOYMENT_RAR:
            System.out.println(" !!!!!!!!!!!!!!!!!!!!! Not implemented !!!!!!!!!!!!!!");
            break;
         case WhereAreYou.DEPLOYMENT_WAR:
            JonasManagementRepr.invoke(JonasObjectName.webContainerService(), "registerWarMBean"
               , asParam, asSignature);
            break;
      }
   }
}
