/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.jonasserver;

import java.io.IOException;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;

/**
 * Test <code>Action</code> that handles events from the tree control test
 * page.
 *
 */

public class EditJmxServerAction extends JonasBaseAction {

// --------------------------------------------------------- Public Methods

   /**
    * Process the specified HTTP request, and create the corresponding HTTP
    * response (or forward to another web component that will create it).
    * Return an <code>ActionForward</code> instance describing where and how
    * control should be forwarded, or <code>null</code> if the response has
    * already been completed.
    *
    * @param mapping The ActionMapping used to select this instance
    * @param actionForm The optional ActionForm bean for this request (if any)
    * @param request The HTTP request we are processing
    * @param response The HTTP response we are creating
    *
    * @exception IOException if an input/output error occurs
    * @exception ServletException if a servlet exception occurs
    */
   public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
      , HttpServletRequest p_Request, HttpServletResponse p_Response)
      throws IOException, ServletException {
      // Form used
      JmxServerForm oForm = (JmxServerForm) p_Form;
      try {
         // Object name used
         ObjectName oObjectName = JonasObjectName.jmxService();
         // Copy scalar properties
         oForm.setMBeanServerId(getStringAttribute(oObjectName, "MBeanServerId"));
         oForm.setSpecificationName(getStringAttribute(oObjectName, "SpecificationName"));
         oForm.setSpecificationVersion(getStringAttribute(oObjectName, "SpecificationVersion"));
         oForm.setSpecificationVendor(getStringAttribute(oObjectName, "SpecificationVendor"));
         oForm.setImplementationName(getStringAttribute(oObjectName, "ImplementationName"));
         oForm.setImplementationVersion(getStringAttribute(oObjectName, "ImplementationVersion"));
         oForm.setImplementationVendor(getStringAttribute(oObjectName, "ImplementationVendor"));
      }
      catch (Throwable t) {
         addGlobalError(t);
         saveErrors(p_Request, m_Errors);
         return (p_Mapping.findForward("Global Error"));
      }

      // Forward to the jsp.
      return (p_Mapping.findForward("JmxServer"));
   }

}
