/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.mbean;

import java.io.IOException;
import java.util.ArrayList;

import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;
import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;

/**
 * List of all Attributes for a MBean.
 *
 */

public final class ListMBeanAttributesAction extends Action {

// --------------------------------------------------------- Public Methods

   /**
    * Process the specified HTTP request, and create the corresponding HTTP
    * response (or forward to another web component that will create it).
    * Return an <code>ActionForward</code> instance describing where and how
    * control should be forwarded, or <code>null</code> if the response has
    * already been completed.
    *
    * @param mapping The ActionMapping used to select this instance
    * @param actionForm The optional ActionForm bean for this request (if any)
    * @param request The HTTP request we are processing
    * @param response The HTTP response we are creating
    *
    * @exception IOException if an input/output error occurs
    * @exception ServletException if a servlet exception occurs
    */
   public ActionForward perform(ActionMapping mapping, ActionForm form, HttpServletRequest request
      , HttpServletResponse response)
      throws IOException, ServletException {

      // The node that was clicked on.
      String nodeLabel = request.getParameter("nodeLabel");
      String nodeSelect = request.getParameter("select");

      // Create a request attribute with our collection of MBeans
      ArrayList list = new ArrayList();
      try {
         // Get all infos of a MBean
         ObjectName on = new ObjectName(nodeSelect);
         MBeanInfo oMBeanInfo = JonasManagementRepr.getMBeanInfo(on);

         // Get attributes infos
         MBeanAttributeInfo[] aoAttributes = oMBeanInfo.getAttributes();
         if (aoAttributes.length > 0) {
            Object oValue = null;
            String sValue = null;
            System.out.println("MBean attributes : " + nodeSelect);
            // Loop to append each attribute node
            for (int i = 0; i < aoAttributes.length; i++) {
               System.out.println(aoAttributes[i].getName() + "\t" + aoAttributes[i].getType());
               try {
                  oValue = JonasManagementRepr.getAttribute(on, aoAttributes[i].getName());
                  sValue = oValue.toString();
               }
               catch (Exception ex) {
                  sValue = "null : " + ex.getMessage();
               }
               list.add(new ViewMBeanAttributes(aoAttributes[i], sValue));
            }
         }
      }
      catch (Exception e) {
         // none
         System.out.println(e.getMessage());
      }
      request.setAttribute("MBeanName", nodeSelect);
      request.setAttribute("MBeanAttributes", list);

      // Forward to the corresponding display page
      return (mapping.findForward("List MBean Attributes"));
   }

// --------------------------------------------------------- Inner Classes

   public class ViewMBeanAttributes {
      private String name;
      private String type;
      private String is;
      private String read;
      private String write;
      private String description;
      private String value;

      public ViewMBeanAttributes() {

      }

      public ViewMBeanAttributes(MBeanAttributeInfo po_Attr, String ps_Value) {
         setName(po_Attr.getName());
         setType(po_Attr.getType());
         setDescription(po_Attr.getDescription());
         setValue(ps_Value);
         if (po_Attr.isIs() == true) {
            setIs("is");
         }
         if (po_Attr.isReadable() == true) {
            setRead("read");
         }
         if (po_Attr.isWritable() == true) {
            setWrite("write");
         }
      }

      public String getName() {
         return name;
      }

      public void setName(String name) {
         this.name = name;
      }

      public String getType() {
         return type;
      }

      public void setType(String type) {
         this.type = type;
      }

      public String getIs() {
         return is;
      }

      public void setIs(String is) {
         this.is = is;
      }

      public String getRead() {
         return read;
      }

      public void setRead(String read) {
         this.read = read;
      }

      public String getWrite() {
         return write;
      }

      public void setWrite(String write) {
         this.write = write;
      }

      public String getDescription() {
         return description;
      }

      public void setDescription(String description) {
         this.description = description;
      }

      public String getValue() {
         return value;
      }

      public void setValue(String value) {
         this.value = value;
      }
   }

}
