/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.mbean;

import java.io.IOException;
import java.util.ArrayList;

import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;
import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;

/**
 * List of all Operations for a MBean.
 *
 */

public final class ListMBeanOperationsAction extends Action {

// --------------------------------------------------------- Public Methods

   /**
    * Process the specified HTTP request, and create the corresponding HTTP
    * response (or forward to another web component that will create it).
    * Return an <code>ActionForward</code> instance describing where and how
    * control should be forwarded, or <code>null</code> if the response has
    * already been completed.
    *
    * @param mapping The ActionMapping used to select this instance
    * @param actionForm The optional ActionForm bean for this request (if any)
    * @param request The HTTP request we are processing
    * @param response The HTTP response we are creating
    *
    * @exception IOException if an input/output error occurs
    * @exception ServletException if a servlet exception occurs
    */
   public ActionForward perform(ActionMapping mapping, ActionForm form, HttpServletRequest request
      , HttpServletResponse response)
      throws IOException, ServletException {

      // The node that was clicked on.
      String nodeLabel = request.getParameter("nodeLabel");
      String nodeSelect = request.getParameter("select");

      // Create a request attribute with our collection of MBeans
      ArrayList list = new ArrayList();
      try {
         // Get all infos of a MBean
         MBeanInfo oMBeanInfo = JonasManagementRepr.getMBeanInfo(new ObjectName(nodeSelect));
         // Get attributes infos
         MBeanOperationInfo[] aoMBeanOperationInfo = oMBeanInfo.getOperations();
         if (aoMBeanOperationInfo.length > 0) {
            // Loop to append each attribute node
            for (int i = 0; i < aoMBeanOperationInfo.length; i++) {
               list.add(new ViewMBeanOperations(aoMBeanOperationInfo[i]));
            }
         }
      }
      catch (Exception e) {
         // none
      }
      // Set the beans
      request.setAttribute("MBeanName", nodeSelect);
      request.setAttribute("MBeanOperations", list);
      // Forward to the corresponding display page
      return (mapping.findForward("List MBean Operations"));
   }

// --------------------------------------------------------- Inner Classes

   public class ViewMBeanOperations {
      private String name;
      private String returnType;
      private String impact;
      private String parameters;
      private String description;

      public ViewMBeanOperations() {
      }

      public ViewMBeanOperations(MBeanOperationInfo po_Ope) {
         setName(po_Ope.getName());
         setReturnType(po_Ope.getReturnType());
         setDescription(po_Ope.getDescription());
         // Impact
         switch (po_Ope.getImpact()) {
            case MBeanOperationInfo.ACTION:
               setImpact("Action");
               break;
            case MBeanOperationInfo.ACTION_INFO:
               setImpact("Action info");
               break;
            case MBeanOperationInfo.INFO:
               setImpact("Info");
               break;
            default:
               setImpact("Unknown");
         }
         // Parameters
         StringBuffer sb = new StringBuffer();
         MBeanParameterInfo[] oParams = po_Ope.getSignature();
         for (int i = 0; i < oParams.length; i++) {
            sb.append(oParams[i].getType());
            sb.append(" ");
            sb.append(oParams[i].getName());
            if (i < (oParams.length - 1)) {
               sb.append(", ");
            }
         }
         setParameters(sb.toString());
      }

      public String getName() {
         return name;
      }

      public void setName(String name) {
         this.name = name;
      }

      public String getReturnType() {
         return returnType;
      }

      public void setReturnType(String returnType) {
         this.returnType = returnType;
      }

      public String getImpact() {
         return impact;
      }

      public void setImpact(String impact) {
         this.impact = impact;
      }

      public String getParameters() {
         return parameters;
      }

      public void setParameters(String parameters) {
         this.parameters = parameters;
      }

      public String getDescription() {
         return description;
      }

      public void setDescription(String description) {
         this.description = description;
      }
   }

}
