/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.mbean;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 * List of all Infos for a MBeans.
 *
 */

public final class ListMBeanPropertiesAction extends Action {

// --------------------------------------------------------- Public Methods

   /**
    * Process the specified HTTP request, and create the corresponding HTTP
    * response (or forward to another web component that will create it).
    * Return an <code>ActionForward</code> instance describing where and how
    * control should be forwarded, or <code>null</code> if the response has
    * already been completed.
    *
    * @param mapping The ActionMapping used to select this instance
    * @param actionForm The optional ActionForm bean for this request (if any)
    * @param request The HTTP request we are processing
    * @param response The HTTP response we are creating
    *
    * @exception IOException if an input/output error occurs
    * @exception ServletException if a servlet exception occurs
    */
   public ActionForward perform(ActionMapping mapping, ActionForm form, HttpServletRequest request
      , HttpServletResponse response)
      throws IOException, ServletException {

      // The node that was clicked on.
      String nodeLabel = request.getParameter("nodeLabel");
      String nodeSelect = request.getParameter("select");

      // Create a request attribute with our collection of MBeans
      String sDomain = null;
      ArrayList list = new ArrayList();
      try {
         // Get all infos of a MBean
         ObjectName on = new ObjectName(nodeSelect);
         sDomain = on.getDomain();

         String sKey;
         String sValue;

         Iterator it = on.getKeyPropertyList().keySet().iterator();
         while (it.hasNext()) {
            sKey = (String) it.next();
            sValue = on.getKeyProperty(sKey);
            list.add(new ViewMBeanProperties(sKey, sValue));
         }
      }
      catch (Exception e) {
         // none
         System.out.println(e.getMessage());
      }
      request.setAttribute("MBeanName", nodeSelect);
      request.setAttribute("MBeanDomain", sDomain);
      request.setAttribute("MBeanProperties", list);

      // Forward to the corresponding display page
      return (mapping.findForward("List MBean Properties"));
   }

// --------------------------------------------------------- Inner Classes

   public class ViewMBeanProperties {
      private String key;
      private String value;

      public ViewMBeanProperties() {

      }

      public ViewMBeanProperties(String ps_Key, String ps_Value) {
         setKey(ps_Key);
         setValue(ps_Value);
      }

      public String getKey() {
         return key;
      }

      public void setKey(String key) {
         this.key = key;
      }

      public String getValue() {
         return value;
      }

      public void setValue(String value) {
         this.value = value;
      }
   }

}
