/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.mbean;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;

/**
 * List of all MBeans that are visible.
 *
 */

public final class ListMBeansAction extends Action {

// --------------------------------------------------------- Public Methods

   /**
    * Process the specified HTTP request, and create the corresponding HTTP
    * response (or forward to another web component that will create it).
    * Return an <code>ActionForward</code> instance describing where and how
    * control should be forwarded, or <code>null</code> if the response has
    * already been completed.
    *
    * @param mapping The ActionMapping used to select this instance
    * @param actionForm The optional ActionForm bean for this request (if any)
    * @param request The HTTP request we are processing
    * @param response The HTTP response we are creating
    *
    * @exception IOException if an input/output error occurs
    * @exception ServletException if a servlet exception occurs
    */
   public ActionForward perform(ActionMapping mapping, ActionForm form, HttpServletRequest request
      , HttpServletResponse response)
      throws IOException, ServletException {

      // Create a request attribute with our collection of MBeans
      ArrayList list = new ArrayList();
      try {
         String sMBeanName;
         ObjectName onMBean;
         Iterator itMBeansNames = JonasAdminJmx.getListMBeanName(null).iterator();
         while (itMBeansNames.hasNext()) {
            sMBeanName = (String) itMBeansNames.next();
            onMBean = new ObjectName(sMBeanName);
            ViewMBeans o = new ViewMBeans();
            o.setDomain(onMBean.getDomain());
            o.setType(onMBean.getKeyProperty("type"));
            o.setName(sMBeanName);
            list.add(o);
         }
      }
      catch (Exception e) {
         // none
      }
      request.setAttribute("MBeans", list);

      // Forward to the corresponding display page
      return (mapping.findForward("List MBeans"));
   }

// --------------------------------------------------------- Inner Classes

   public class ViewMBeans {
      private String domain;
      private String type;
      private String name;
      public String getDomain() {
         return domain;
      }

      public void setDomain(String domain) {
         this.domain = domain;
      }

      public String getType() {
         return type;
      }

      public void setType(String type) {
         this.type = type;
      }

      public String getName() {
         return name;
      }

      public void setName(String name) {
         this.name = name;
      }
   }

}
