/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.container;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;
import org.objectweb.jonas.webapp.jonasadmin.service.ejb.EjbItem;
import org.objectweb.jonas.webapp.jonasadmin.service.ejb.EjbItemByNameComparator;

/**
 *
 */

public class EditContainerAction extends JonasBaseAction {

// --------------------------------------------------------- Public Methods

   /**
    */
   public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
      , HttpServletRequest p_Request, HttpServletResponse p_Response)
      throws IOException, ServletException {
      // Form used
      ContainerForm oForm = (ContainerForm) p_Form;
      try {
         // Selected container
         String sSelect = p_Request.getParameter("select");
         // Object name used
         ObjectName oObjectName = JonasObjectName.container(sSelect);
         oForm.setFileName(getStringAttribute(oObjectName, "FileName"));
         oForm.setContainerName(getStringAttribute(oObjectName, "ContainerName"));
         oForm.setCurrentNumberOfMDBType(getIntegerAttribute(oObjectName, "CurrentNumberOfMDBType"));
         oForm.setCurrentNumberOfSBFType(getIntegerAttribute(oObjectName, "CurrentNumberOfSBFType"));
         oForm.setCurrentNumberOfBMPType(getIntegerAttribute(oObjectName, "CurrentNumberOfBMPType"));
         oForm.setCurrentNumberOfSBLType(getIntegerAttribute(oObjectName, "CurrentNumberOfSBLType"));
         oForm.setCurrentNumberOfCMPType(getIntegerAttribute(oObjectName, "CurrentNumberOfCMPType"));
         oForm.setCurrentNumberOfBeanType(getIntegerAttribute(oObjectName
            , "CurrentNumberOfBeanType"));
         // Prepare Ejb list
         searchEjbs(oForm, sSelect);
      }
      catch (Throwable t) {
         addGlobalError(t);
         saveErrors(p_Request, m_Errors);
         return (p_Mapping.findForward("Global Error"));
      }
      // Forward to the jsp.
      return (p_Mapping.findForward("Container"));
   }

   protected void searchEjbs(ContainerForm p_Form, String p_Filename)
      throws Exception {
      String sName;
      String sTypeResource;
      Iterator itNames;
      ArrayList al = new ArrayList();
      // get BMP (Entity with Bean-Managed Persistence)
      itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbBmp(p_Filename)).iterator();
      while (itNames.hasNext()) {
         sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
         sTypeResource = m_Resources.getMessage("ejb.fulltype." + EjbItem.EJB_BMP_STRING);
         al.add(new EjbItem(EjbItem.EJB_BMP, sName, p_Filename, sTypeResource));
      }
      // get CMP (Entity with Container-Managed Persistence)
      itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbCmp(p_Filename)).iterator();
      while (itNames.hasNext()) {
         sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
         sTypeResource = m_Resources.getMessage("ejb.fulltype." + EjbItem.EJB_CMP_STRING);
         al.add(new EjbItem(EjbItem.EJB_CMP, sName, p_Filename, sTypeResource));
      }
      // get SBF (Stateful Session Bean)
      itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbSbf(p_Filename)).iterator();
      while (itNames.hasNext()) {
         sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
         sTypeResource = m_Resources.getMessage("ejb.fulltype." + EjbItem.EJB_SBF_STRING);
         al.add(new EjbItem(EjbItem.EJB_SBF, sName, p_Filename, sTypeResource));
      }
      // get SBL (Stateless Session Bean)
      itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbSbl(p_Filename)).iterator();
      while (itNames.hasNext()) {
         sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
         sTypeResource = m_Resources.getMessage("ejb.fulltype." + EjbItem.EJB_SBL_STRING);
         al.add(new EjbItem(EjbItem.EJB_SBL, sName, p_Filename, sTypeResource));
      }
      // get MDB (Message Driven Bean)
      itNames = JonasAdminJmx.getListMBeanName(JonasObjectName.allEjbMdb(p_Filename)).iterator();
      while (itNames.hasNext()) {
         sName = JonasAdminJmx.extractValueMbeanName(",name", itNames.next().toString());
         sTypeResource = m_Resources.getMessage("ejb.fulltype." + EjbItem.EJB_MDB_STRING);
         al.add(new EjbItem(EjbItem.EJB_MDB, sName, p_Filename, sTypeResource));
      }

      // Sort by name
      Collections.sort(al, new EjbItemByNameComparator());
      // Save in form
      p_Form.setEjbs(al);
   }
}
