/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.ejb;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;

/**
 *
 */

abstract public class EditEjbAction extends JonasBaseAction {

// --------------------------------------------------------- Protected variables

   protected String ms_ParamType = null;
   protected String ms_ParamName = null;
   protected String ms_ParamFile = null;
   protected String ms_ParamNode = null;

// --------------------------------------------------------- Public Methods

   /**
    * Execute a action.
    *
    * @param p_Mapping <code>ActionForward</code> instance
    * @param p_Form <code>ActionForm</code> instance
    * @param p_Request <code>HttpServletRequest</code> instance
    * @param p_Response <code>HttpServletResponse</code> instance
    * @return <code>ActionForward</code> instance
    * @throws IOException
    * @throws ServletException
    */
   public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
      , HttpServletRequest p_Request, HttpServletResponse p_Response)
      throws IOException, ServletException {

      // Get Http parameters
      ms_ParamType = p_Request.getParameter("type");
      ms_ParamName = p_Request.getParameter("name");
      ms_ParamFile = p_Request.getParameter("file");
      ms_ParamNode = p_Request.getParameter("node");

      // Form used
      EjbForm oForm = getEjbForm();
      m_Session.setAttribute("ejbForm", oForm);

      // Put global properties
      oForm.setType(ms_ParamType);
      oForm.setName(ms_ParamName);
      oForm.setFile(ms_ParamFile);
      oForm.setNode(ms_ParamNode);
      if (ms_ParamType != null) {
         String sFullType = m_Resources.getMessage("ejb.fulltype." + ms_ParamType);
         oForm.setFullType(sFullType);
      }

      try {
         // Object name used
         ObjectName oObjectName = getEjbObjectName();
         // Fill Ejb infos
         fillEjbInfo(oForm, oObjectName);
         // Fill Ejb dependencies (Datasource, JMS, ...)
         fillEjbDependencies(oForm, oObjectName);
      }
      catch (Throwable t) {
         addGlobalError(t);
         saveErrors(p_Request, m_Errors);
         return (p_Mapping.findForward("Global Error"));
      }
      // Forward to the jsp.
      return (p_Mapping.findForward(getEjbForward()));
   }

// --------------------------------------------------------- Protected Methods

   /**
    * The global forward to go.
    *
    * @return Forward
    */
   protected String getEjbForward() {
      return "Ejb";
   }

   /**
    * Return a <code>EjbForm</code> instance associate to the EJB.
    *
    * @return A form instance
    */
   abstract protected EjbForm getEjbForm();

   /**
    * Fill all infos of EJB in the <code>EjbForm</code> instance.
    *
    * @param p_Form Instance to fill
    * @param p_ObjectName Instance to get infos
    * @throws Exception
    */
   abstract protected void fillEjbInfo(EjbForm p_Form, ObjectName p_ObjectName)
      throws Exception;

   /**
       * Get a <code>ObjectName</code> instance for the Ejb given with the parameters of the HTPP request.
    *
    * @return The ObjectName instance
    * @throws Exception
    */
   abstract protected ObjectName getEjbObjectName()
      throws Exception;

   /**
    * Fill all global infos of EJB in the <code>EjbForm</code> instance.
    *
    * @param p_Form Instance to fill
    * @param p_ObjectName Instance to get infos
    * @throws Exception
    */
   protected void fillEjbGlobalInfo(EjbForm p_Form, ObjectName p_ObjectName)
      throws Exception {

      p_Form.setCurrentInstancePoolSize(getIntegerAttribute(p_ObjectName, "CurrentInstancePoolSize"));
      p_Form.setDisplayName(getStringAttribute(p_ObjectName, "DisplayName"));
      p_Form.setEjbClass(getStringAttribute(p_ObjectName, "EJBClass"));
      p_Form.setEjbFileName(getStringAttribute(p_ObjectName, "EJBFileName"));
      p_Form.setEjbName(getStringAttribute(p_ObjectName, "EJBName"));
      p_Form.setHomeClass(getStringAttribute(p_ObjectName, "HomeClass"));
      p_Form.setJndiName(getStringAttribute(p_ObjectName, "JndiName"));
      p_Form.setLocalClass(getStringAttribute(p_ObjectName, "LocalClass"));
      p_Form.setLocalHomeClass(getStringAttribute(p_ObjectName, "LocalHomeClass"));
      p_Form.setRemoteClass(getStringAttribute(p_ObjectName, "RemoteClass"));
   }

   /**
    * Fill all dependencies of EJB in the <code>EjbForm</code> instance.
    *
    * @param p_Form Instance to fill
    * @param p_ObjectName Instance to get infos
    * @throws Exception
    */
   protected void fillEjbDependencies(EjbForm p_Form, ObjectName p_ObjectName)
      throws Exception {
      // Dependency with Datasource service
      p_Form.setDatabaseServiceActivated(JonasManagementRepr.isRegistered(JonasObjectName.
         databaseService()));
      HashMap hDataSources = new HashMap((Map) JonasManagementRepr.getAttribute(p_ObjectName
         , "AllDataSourceName"));
      if (hDataSources.size() > 0) {
         p_Form.setDataSource(true);
         p_Form.setDataSources(hDataSources);
      }
      // Dependency with JMS service
      p_Form.setJmsServiceActivated(JonasManagementRepr.isRegistered(JonasObjectName.jmsService()));
      ArrayList alJmsConnections = new ArrayList((Collection) JonasManagementRepr.getAttribute(
         p_ObjectName, "AllJMSConnectionFactoryName"));
      if (alJmsConnections.size() > 0) {
         p_Form.setJmsConnection(true);
         p_Form.setJmsConnections(alJmsConnections);
      }
      ArrayList alJmsDestinations = new ArrayList((Collection) JonasManagementRepr.getAttribute(
         p_ObjectName, "AllJMSDestinationName"));
      if (alJmsDestinations.size() > 0) {
         p_Form.setJmsDestination(true);
         p_Form.setJmsDestinations(alJmsDestinations);
      }
      // Dependency with Mail service
      p_Form.setMailServiceActivated(JonasManagementRepr.isRegistered(JonasObjectName.mailService()));
      HashMap hMailSessions = new HashMap((Map) JonasManagementRepr.getAttribute(p_ObjectName
         , "AllMailFactorySName"));
      if (hMailSessions.size() > 0) {
         p_Form.setMailSession(true);
         p_Form.setMailSessions(hMailSessions);
      }
      HashMap hMailMimes = new HashMap((Map) JonasManagementRepr.getAttribute(p_ObjectName
         , "AllMailFactoryMName"));
      if (hMailMimes.size() > 0) {
         p_Form.setMailMime(true);
         p_Form.setMailMimes(hMailMimes);
      }
      // Detect one dependency
      p_Form.setDependency(p_Form.isDataSource() || p_Form.isJmsConnection()
         || p_Form.isJmsDestination() || p_Form.isMailSession() || p_Form.isMailMime());
   }
}
