/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.service.ejb;

import javax.management.ObjectName;

import org.objectweb.jonas.jmx.JonasManagementRepr;

/**
 *
 */

abstract public class EditEjbEntityAction extends EditEjbAction {
// --------------------------------------------------------- Public Methods

// --------------------------------------------------------- Protected Methods

   /**
    * Return a <code>EjbEntityForm</code> instance associate to the EJB.
    *
    * @return A form instance
    */
   protected EjbForm getEjbForm() {
      return new EjbEntityForm();
   }

   /**
    * Fill all infos of EJB Entity in the <code>EjbEntityForm</code> instance.
    *
    * @param p_Form Instance to fill
    * @param p_ObjectName Instance to get infos
    * @throws Exception
    */
   protected void fillEjbInfo(EjbForm p_Form, ObjectName p_ObjectName)
      throws Exception {
      fillEjbGlobalInfo(p_Form, p_ObjectName);

      EjbEntityForm oForm = (EjbEntityForm) p_Form;
      oForm.setCacheSize(getIntegerAttribute(p_ObjectName, "CacheSize"));
      oForm.setMaxCacheSize(getIntegerAttribute(p_ObjectName, "MaxCacheSize"));
      oForm.setMinPoolSize(getIntegerAttribute(p_ObjectName, "MinPoolSize"));
      oForm.setPassivationTimeOut(getLongAttribute(p_ObjectName, "PassivationTimeOut"));
      oForm.setPoolSize(getIntegerAttribute(p_ObjectName, "PoolSize"));
      oForm.setShared(getBooleanAttribute(p_ObjectName, "Shared"));

      Integer[] aiEntityCounters = (Integer[]) JonasManagementRepr.getAttribute(p_ObjectName
         , "EntityCounters");
      oForm.setUsedInTxInstance(aiEntityCounters[0].intValue());
      oForm.setUsedOutTxInstance(aiEntityCounters[1].intValue());
      oForm.setUnusedInstance(aiEntityCounters[2].intValue());
      oForm.setPassivatedInstance(aiEntityCounters[3].intValue());
      oForm.setRemovedInstance(aiEntityCounters[4].intValue());
   }

   /**
    * The global forward to go.
    *
    * @return Forward
    */
   protected String getEjbForward() {
      return "Ejb Entity";
   }
}
