/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.taglib;

import javax.servlet.jsp.JspException;

import org.apache.struts.taglib.html.LinkTag;
import org.apache.struts.util.ResponseUtils;

import org.objectweb.jonas.webapp.jonasadmin.WhereAreYou;

public class JonasButtonTag extends LinkTag {

// ----------------------------------------------------- Manifest Constants

   private static final String s_ImageBorder = "dot.gif";

// ----------------------------------------------------- Instance Variables

   protected String ms_PathImage = null;
   protected String ms_LastStyleClass = null;
   protected String ms_StyleClassButton = "btn";

// ------------------------------------------------------------- Properties

   private int widthBorder = 1;
   private int heightBorder = 1;
   private String imagesDir = null;
   private String styleClassBorderLight = "btnBorderLight";
   private String styleClassBorderShadow = "btnBorderShadow";
   private String styleClassBackground = "btn";
   private String styleClassDisabled = "btnDisabled";

   public int getHeightBorder() {
      return heightBorder;
   }

   public void setHeightBorder(int heightBorder) {
      this.heightBorder = heightBorder;
   }

   public String getImagesDir() {
      return imagesDir;
   }

   public void setImagesDir(String imagesDir) {
      this.imagesDir = imagesDir;
   }

   public int getWidthBorder() {
      return widthBorder;
   }

   public void setWidthBorder(int widthBorder) {
      this.widthBorder = widthBorder;
   }

   public String getStyleClassBorderLight() {
      return styleClassBorderLight;
   }

   public void setStyleClassBorderLight(String styleClassBorderLight) {
      this.styleClassBorderLight = styleClassBorderLight;
   }

   public String getStyleClassBorderShadow() {
      return styleClassBorderShadow;
   }

   public void setStyleClassBorderShadow(String styleClassBorderShadow) {
      this.styleClassBorderShadow = styleClassBorderShadow;
   }

   public String getStyleClassBackground() {
      return styleClassBackground;
   }

   public void setStyleClassBackground(String styleClassBackground) {
      this.styleClassBackground = styleClassBackground;
   }

   public String getStyleClassDisabled() {
      return styleClassDisabled;
   }

   public void setStyleClassDisabled(String styleClassDisabled) {
      this.styleClassDisabled = styleClassDisabled;
   }

// ------------------------------------------------------------- Public methods

   /**
    * Render the beginning of the hyperlink.
    *
    * @exception JspException if a JSP exception has occurred
    */
   public int doStartTag()
      throws JspException {
      // Force style class to the Link Tag
      ms_LastStyleClass = getStyleClass();
      if (getStyleClass() == null) {
         setStyleClass(ms_StyleClassButton);
      }
      // Prepare path image
      makePathImage();
      // Add the begining of render button
      StringBuffer sb = new StringBuffer("<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">");
      sb.append(prepareBorderTop());
      sb.append("<tr>");
      sb.append(prepareBorderColumn(styleClassBorderLight));
      sb.append("<td class=\"");
      if (getDisabled() == false) {
         sb.append(styleClassBackground);
      }
      else {
         sb.append(styleClassDisabled);
      }
      sb.append("\">");
      ResponseUtils.write(pageContext, sb.toString());

      if (getDisabled() == false) {
         // Add link if enabled
         return (super.doStartTag());
      }

      return (EVAL_BODY_BUFFERED);
   }

   /**
    * Render the end of the hyperlink.
    *
    * @exception JspException if a JSP exception has occurred
    */
   public int doEndTag()
      throws JspException {
      StringBuffer sb = new StringBuffer();
      int iRet = EVAL_PAGE;

      // If body text exist pre-fix and post-fix with space
      if (text != null) {
         StringBuffer sbText = new StringBuffer("&nbsp;");
         sbText.append(text);
         sbText.append("&nbsp;");
         text = sbText.toString();
      }

      if (getDisabled() == false) {
         // Add link if enabled
         iRet = super.doEndTag();
      }
      else {
         sb.append(text);
      }

      // Add the end of render button
      sb.append("</td>");
      sb.append(prepareBorderColumn(styleClassBorderShadow));
      sb.append("</tr>");
      sb.append(prepareBorderBottom());
      sb.append("</table>");
      ResponseUtils.write(pageContext, sb.toString());

      // Force last style
      setStyleClass(ms_LastStyleClass);
      ms_LastStyleClass = null;

      return (iRet);
   }

   public String getImagesRoot() {
      String sImagesRoot = null;
      WhereAreYou oWhere = getWhereAreYouInstance();
      if (oWhere != null) {
         sImagesRoot = oWhere.getImagesRoot();
      }
      return sImagesRoot;
   }

   public void release() {
      super.release();
      ms_PathImage = null;
      imagesDir = null;
      ms_StyleClassButton = null;
      styleClassBorderLight = null;
      styleClassBorderShadow = null;
      styleClassBackground = null;
      styleClassDisabled = null;
   }

// --------------------------------------------------------- Protected Methods
   protected void makePathImage() {
      // Prepare image separator
      if (imagesDir != null) {
         ms_PathImage = imagesDir + "/" + s_ImageBorder;
      }
      else if (getImagesRoot() != null) {
         ms_PathImage = getImagesRoot() + "/" + s_ImageBorder;
      }
      else {
         ms_PathImage = s_ImageBorder;
      }
   }

   /**
    * Return render image.
    * @return
    */
   protected String prepareImage() {
      StringBuffer sb = new StringBuffer("<img src=\"");
      sb.append(ms_PathImage);
      sb.append("\" width=\"");
      sb.append(widthBorder);
      sb.append("\" height=\"");
      sb.append(heightBorder);
      sb.append("\" border=\"0\">");
      return sb.toString();
   }

   /**
    * Return render image.
    * @return
    */
   protected String prepareBorderColumn(String ps_Class) {
      StringBuffer sb = new StringBuffer("<td width=\"");
      sb.append(widthBorder);
      sb.append("\" height=\"");
      sb.append(heightBorder);
      sb.append("\" class=\"");
      //btnBorderLight
      sb.append(ps_Class);
      sb.append("\">");
      sb.append(prepareImage());
      sb.append("</td>");
      return sb.toString();
   }

   protected String prepareBorderTop() {
      StringBuffer sb = new StringBuffer("<tr>");
      sb.append(prepareBorderColumn(styleClassBorderLight));
      sb.append(prepareBorderColumn(styleClassBorderLight));
      sb.append(prepareBorderColumn(styleClassBorderLight));
      sb.append("</tr>");
      return sb.toString();
   }

   protected String prepareBorderBottom() {
      StringBuffer sb = new StringBuffer("<tr>");
      sb.append(prepareBorderColumn(styleClassBorderLight));
      sb.append(prepareBorderColumn(styleClassBorderShadow));
      sb.append(prepareBorderColumn(styleClassBorderShadow));
      sb.append("</tr>");
      return sb.toString();
   }

   protected WhereAreYou getWhereAreYouInstance() {
      return (WhereAreYou) pageContext.getSession().getAttribute("WhereAreYou");
   }
}