/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.taglib;

import java.net.MalformedURLException;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.Tag;

import org.apache.struts.util.RequestUtils;
import org.apache.webapp.admin.TreeControlNode;

public class TabTag extends BodyTagSupport {

// ----------------------------------------------------- Instance Variables
   private String m_Body = null;
   private String m_Url = null;

// ----------------------------------------------------- Properties

   private boolean selected = false;
   private String href = null;
   private String forward = null;
   private String forwardControl = null;
   /**
    * Accessor Href property.
    */
   public String getHref() {
      return href;
   }

   public void setHref(String href) {
      this.href = href;
   }

   /**
    * Accessor Selected property.
    */
   public boolean getSelected() {
      return (this.selected);
   }

   public void setSelected(boolean selected) {
      this.selected = selected;
   }

   /**
    * Accessor Forward property.
    */
   public String getForward() {
      return forward;
   }

   public void setForward(String forward) {
      this.forward = forward;
   }

   /**
    * Accessor ForwardControl property.
    */
   public String getForwardControl() {
      return forwardControl;
   }

   public void setForwardControl(String forwardControl) {
      this.forwardControl = forwardControl;
   }

// --------------------------------------------------------- Public Methods

   public int doStartTag()
      throws JspException {
      // Initialize the older body
      m_Body = null;

      // Do no further processing for now
      return (EVAL_BODY_BUFFERED);
   }

   public int doAfterBody()
      throws JspException {
      String sBody = bodyContent.getString();
      if (sBody != null) {
         sBody = sBody.trim();
         if (sBody.length() > 0) {
            this.m_Body = sBody;
         }
      }
      return (SKIP_BODY);
   }

   /**
    * Record this tab with our surrounding TabsTag instance.
    *
    * @exception JspException if a processing error occurs
    */
   public int doEndTag()
      throws JspException {
      // Initialise URL
      m_Url = null;
      try {
         m_Url = RequestUtils.computeURL(pageContext, forward, href, null, null, null, false);
      }
      catch (MalformedURLException e) {
         // none
      }

      // Find our parent TabsTag instance
      Tag parent = findAncestorWithClass(this, TabsTag.class);
      if ((parent == null) || !(parent instanceof TabsTag)) {
         throw new JspException("Must be nested in a TabsTag instance");
      }
      TabsTag oTabs = (TabsTag) parent;

      // Verify forwardControl parameter
      if ((m_Url == null) && (forwardControl != null) && (oTabs.isUsingWhere() == true)) {
         try {
            // Get the selected node
            TreeControlNode oNode = oTabs.getSelectedTreeControlNode();
            if (oNode != null) {
               try {
                  String sForward = RequestUtils.computeURL(pageContext, forwardControl, null, null, null, null, false);
                  int iPos = oNode.getAction().indexOf("?");
                  m_Url = sForward + oNode.getAction().substring(iPos);
               }
               catch (MalformedURLException e) {
                  // none
               }
            }
         }
         catch (Exception ex) {
            // none
         }
      }
      // Verify Url
      if (m_Url == null) {
         m_Url = new String("");
      }
      // Verify body label
      if (m_Body == null) {
         m_Body = new String("");
      }
      // Register the information for the action represented by
      // this action
      HttpServletResponse response = (HttpServletResponse) pageContext.getResponse();
      oTabs.addTab(m_Body, m_Url, selected);

      return (EVAL_PAGE);
   }

   /**
    * Release all state information set by this tag.
    */
   public void release() {
      this.m_Body = null;
      this.href = null;
      this.forward = null;
      this.forwardControl = null;
   }
}
