/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D, Bull
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.deployment.api;

import java.util.Iterator;

import org.objectweb.jonas_ejb.deployment.xml.*;


/**
 * Class to hold meta-information related to an CMP v2 entity bean with jdbc data store.
 * @author Christophe Ney [cney@batisseurs.com]
 * contributor(s):
 * - Helene Joanin on May 2003: code cleanup
 * - Helene Joanin on May 2003: complement for legacy first version 
 */

// TODO: Review this class, many methods are common with EntityJdbcCmp1Desc

public class EntityJdbcCmp2Desc extends EntityCmp2Desc {

    protected String dsname;
    protected String jdbcTableName = null;
    protected String jdbcAutomaticPk;

    /**
     * constructor: called when the DeploymentDescriptor is read.
     * Currently, called by both GenIC and createContainer.
     */
    public EntityJdbcCmp2Desc(ClassLoader classLoader,
                              Entity ent,
                              AssemblyDescriptor asd,
                              JonasEntity jEnt,
                              DeploymentCmp2Desc dc2d)
        throws DeploymentDescException {

        super(classLoader, ent, asd, jEnt, dc2d);

        // check for jdbcMapping
        JdbcMapping jm = jEnt.getJdbcMapping();
        if (jm == null) {
            throw new DeploymentDescException("jdbc-mapping missing for bean " + ent.getEjbName());
        }

        // jndi name of the datasource
        dsname = jm.getJndiName().getValue();

        // jdbc table name
        if (jm.getJdbcTableName() != null) {
            if (jm.getJdbcTableName().getValue().length() != 0) {
                jdbcTableName = jm.getJdbcTableName().getValue();
            }
        }
        if (jdbcTableName == null) {
            // Default value
            jdbcTableName = getAbstractSchemaName().toUpperCase() + "_";
        }

        // Default mapping information for cmp fields
        for (Iterator i = fieldDesc.keySet().iterator();i.hasNext();) {
            String fn = (String) i.next();
            ((FieldJdbcDesc)(fieldDesc.get(fn))).setJdbcFieldName(fn + "_");
        }
        // mapping information for cmp fields from jonas DD
        for (Iterator i = jm.getCmpFieldJdbcMappingList().iterator();i.hasNext();) {
            CmpFieldJdbcMapping fm = (CmpFieldJdbcMapping) i.next();
            String fn = fm.getFieldName().getValue();
            String cn = fm.getJdbcFieldName().getValue();
            String ct = null;
            if (fm.getSqlType() != null) {
                ct = fm.getSqlType().getValue();
            }
            FieldJdbcDesc fdesc = (FieldJdbcDesc) fieldDesc.get(fn);
            fdesc.setJdbcFieldName(cn);
            if (ct != null) {
                fdesc.setSqlType(ct);
            }
        }

        // automatic-pk
        if (jm.getAutomaticPk() != null) {
            // optionnal parameter automatic-pk 
            jdbcAutomaticPk = jm.getAutomaticPk().getValue();
        } else {
            jdbcAutomaticPk = "false"; // default value
        }

        // Check if prim-key-class type is Integer
        String primkeytype = ent.getPrimKeyClass().getValue();
        if (this.isAutomaticPk()) {
            if (!(primkeytype.equalsIgnoreCase("java.lang.Integer") || primkeytype.equalsIgnoreCase("Integer"))) {
                throw new DeploymentDescException("You must used type java.lang.Integer for your automatic primary key field for bean " + ent.getEjbName().getValue());
            }
        }

    }

    /**
     * field descriptor factory method
     */
    protected FieldDesc newFieldDescInstance() {
        return new FieldJdbcDesc();
    }

    /**
     * Get the datasource jndi name
     * @return String representation of the jndi name
     */
    public String getDatasourceJndiName() {
        return dsname;
    }

    /**
     * Get the associated DataBase table name.
     * @return Name of the database table where entity bean is stored
     */
    public String getJdbcTableName() {
        return jdbcTableName;
    }

    /**
     * Assessor for existence of automatic-pk element to True value
     * @param field public field of the bean class
     * @return true if automatic-pk element value is true else otherwise false
     */
    public boolean isAutomaticPk() {
        return jdbcAutomaticPk.equalsIgnoreCase("True");
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        ret.append("\ngetDatasourceJndiName()=" + getDatasourceJndiName());
        ret.append("\ngetJdbcTableName()=" + getJdbcTableName());
        return ret.toString();
    }


}
