/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_jms;

import fr.dyade.aaa.agent.AgentServer;
import fr.dyade.aaa.joram.admin.AdminException;
import javax.jms.ConnectionFactory;
import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;
import javax.jms.Topic;
import javax.jms.TopicConnectionFactory;
import javax.jms.XAConnectionFactory;
import javax.jms.XAQueueConnectionFactory;
import javax.jms.XATopicConnectionFactory;
import javax.naming.InitialContext; 
import javax.naming.NamingException;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import org.objectweb.jonas_jms.api.JmsAdministration;
import org.objectweb.common.Env;

import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Joram Administration. This Joram specific class implements
 * the JMS independant interface JmsAdministration.
 * @author Philippe Coq
 * Contributor(s): 
 *  Philippe Durieux
 *  Jeff Mesnil: for JORAM 3.x integration
 *  Frederic Maistre for JORAM 4.x integration
 */
public class JmsAdminForJoram implements JmsAdministration {

    private static String QUEUE_CONN_FACT_NAME="JQCF";
    private static String TOPIC_CONN_FACT_NAME="JTCF";
    private static String CONN_FACT_NAME="JCF";
    private boolean collocatedmom = false;
    private InitialContext ictx = null;
    private Vector namelist = new Vector();
    private XAConnectionFactory xacf = null;
    private XATopicConnectionFactory xatcf = null;
    private XAQueueConnectionFactory xaqcf = null;
    private ConnectionFactory jcf = null;
    private TopicConnectionFactory jtcf = null;
    private QueueConnectionFactory jqcf = null;
    private fr.dyade.aaa.joram.admin.AdminItf admin = null;
    private fr.dyade.aaa.joram.admin.User user = null;
    private fr.dyade.aaa.joram.admin.User jonasUser = null;
    private String host = null;
    private int port;

    /**
     * default constructor.
     * This class will be created with newInstance().
     */
    public JmsAdminForJoram() {
    }

    // ------------------------------------------------------------
    // private methods
    // ------------------------------------------------------------

    /**
     * start the MOM in the same JVM than the current
     *
     * Start the A3 AgentServer with the default configuration
     * in transient mode and with the a3servers.xml file located into $JONAS_BASE/conf
     * what it is made here is equivalent to
     *  java -Djonas.base=$JONAS_BASE -DTransaction=fr.dyade.aaa.util.NullTransaction
     *       -Dfr.dyade.aaa.agent.A3CONF_DIR=$JONAS_ROOT/config
     *           fr.dyade.aaa.agent.AgentServer 0 ./s0&
     */
    private void startMOM() throws Exception {
        TraceJms.logger.log(BasicLevel.DEBUG,"");

        try {
            // -DTransaction=fr.dyade.aaa.util.NullTransaction
            String TRANSACTION_PROP = "Transaction";
            String s1 = System.getProperty(TRANSACTION_PROP);
            if (s1 == null) {
                System.setProperty(TRANSACTION_PROP, "fr.dyade.aaa.util.NullTransaction");
            }

            // -Dfr.dyade.aaa.agent.A3CONF_DIR=$JONAS_BASE/conf
            String A3CONF_DIR_PROP = "fr.dyade.aaa.agent.A3CONF_DIR";
            String a3config_dir = System.getProperty(A3CONF_DIR_PROP);
            if (a3config_dir == null) {
                System.setProperty(A3CONF_DIR_PROP, System.getProperty("jonas.base")
                                   + System.getProperty("file.separator") + "conf");
            }
            if(TraceJms.isDebug())
                TraceJms.logger.log(BasicLevel.DEBUG,"a3config_dir= "+a3config_dir);
            // java fr.dyade.aaa.agent.AgentServer 0 s0
            // 0 identifies wich server to run among those described in config file.
            // s0 is a directory used for persistence.
            String []args = new String[2];
            args[0] = "0";
            args[1] = "s0";
            AgentServer.init(args);
            AgentServer.start();

            // wait 10 millisec for the MOM to start ???
            Thread.sleep(10);
	    
            // Get host and port that must be used to start Admin
            // mainly to get the port number set in a3servers.xml
            short sid = AgentServer.getServerId();	// should be 0
            host = AgentServer.getHostname(sid);	// should be localhost
            port = 16010;
            try {
                String s = AgentServer.getServiceArgs(sid, "fr.dyade.aaa.mom.proxies.tcp.ConnectionFactory");
                StringTokenizer st = new StringTokenizer(s);
                port = Integer.parseInt(st.nextToken());
            } catch (NumberFormatException exc) {
                TraceJms.logger.log(BasicLevel.ERROR,"exception: "+exc);
            } catch (Exception e) {
                TraceJms.logger.log(BasicLevel.ERROR,"MOM exception:"+e);
                throw e;
            }
            TraceJms.logger.log(BasicLevel.WARN, "starting MOM on host " + host + ", port " + port);

        } catch (Exception ex) {
            TraceJms.logger.log(BasicLevel.ERROR,"exception: "+ex);
            throw ex;
        }
    }
    
    // ------------------------------------------------------------
    // JmsAdministration implementation
    // ------------------------------------------------------------

    /**
     * Jms Administrator is created with newInstance().
     * initialization is done later with this method.
     * The MOM will be started if collocated.
     * This method should create an XATopicConnectionFactory and
     * an XAQueueConnectionFactory
     * @param boolean true for if the MOM in run in the current JVM
     * @param String url connexion that must be used if not collocated
     */
    public void start(boolean collocated, String url) throws Exception {

        // Start the MOM in this JVM if collocated.
        if (collocated) {
            collocatedmom = true;
            startMOM();
        }

        // Create an InitialContext
        ictx = new InitialContext();

        if (admin == null) {
            admin = new fr.dyade.aaa.joram.admin.AdminImpl();
            if (host != null) {
                // collocated: use url got from the MOM
                TraceJms.logger.log(BasicLevel.WARN, "starting JmsAdmin with host " + host + ", port " + port);
                admin.connect(host, port, "root", "root", 100);
            } else if (url != null && url.length() > 0) {
                // not collocated: use url from jonas.properties
                int indexOfHost = url.indexOf("//") + 2;
                int indexOfPort = url.indexOf(":", indexOfHost) + 1;
                host = url.substring(indexOfHost, indexOfPort - 1);
                port = Integer.parseInt(url.substring(indexOfPort));
                TraceJms.logger.log(BasicLevel.WARN, "starting JmsAdmin with host " + host + ", port " + port);
                admin.connect(host, port, "root", "root", 100);
            } else {
                // use default url
                TraceJms.logger.log(BasicLevel.WARN, "starting JmsAdmin with default url");
                admin.connect("root", "root", 100);
            }
        }

        user = admin.createUser("anonymous", "anonymous", 0);
        jonasUser = admin.createUser("jonas", "jonas", 0);

        // Create connection factories  that will be used by the EJB components.
        xacf = admin.createXAConnectionFactory();
        xatcf = admin.createXATopicConnectionFactory();
        xaqcf = admin.createXAQueueConnectionFactory();


        // Create connection factories  that will be used by a pure JMS Client
        String name = CONN_FACT_NAME;
        try {
            jcf = admin.createConnectionFactory();
            ictx.rebind(name, jcf);
            namelist.addElement(name);
        } catch (NamingException e) {
            TraceJms.logger.log(BasicLevel.ERROR,"cannot register "+name);
        }

        name = QUEUE_CONN_FACT_NAME;
        try {
            jqcf = admin.createQueueConnectionFactory();
            ictx.rebind(name, jqcf);
            namelist.addElement(name);
        } catch (NamingException e) {
            TraceJms.logger.log(BasicLevel.ERROR,"cannot register "+name);
        }

        name = TOPIC_CONN_FACT_NAME;
        try {
            jtcf = admin.createTopicConnectionFactory();
            ictx.rebind(name, jtcf);
            namelist.addElement(name);
        } catch (NamingException e) {
            TraceJms.logger.log(BasicLevel.ERROR,"cannot register "+name);
        }
    }

    /**
     * Stop the Jms Administrator
     */
    public void stop() {
        if ( Env.isJAVA4() ){
                // workaround against bug in SUN JVM JDK1.3 on Linux
            if (admin != null) {            
                try {               
                    admin.disconnect(); 
                } catch  (Exception ex) {
                    TraceJms.logger.log(BasicLevel.ERROR,"problem for closing admin object "+ex);
                }
            }
            
            if (collocatedmom) {
                AgentServer.stop(); 
                TraceJms.logger.log(BasicLevel.DEBUG,"agentServer stopped");
            }
        }
        try {		    
            ictx.unbind(CONN_FACT_NAME);
            ictx.unbind(QUEUE_CONN_FACT_NAME);
            ictx.unbind(TOPIC_CONN_FACT_NAME);
            TraceJms.logger.log(BasicLevel.DEBUG,"unbind connection factories");
        } catch(Exception ex) {
            TraceJms.logger.log(BasicLevel.ERROR,"cannot unbind connection factories");
        }
    }


    /**
     * Get the XAConnectionFactory
     */
    public XAConnectionFactory getXAConnectionFactory() {
        return xacf;
    }

    /**
     * Get the XATopicConnectionFactory
     */
    public XATopicConnectionFactory getXATopicConnectionFactory() {
        return xatcf;
    }

    /**
     * Get the XAQueueConnectionFactory
     */
    public XAQueueConnectionFactory getXAQueueConnectionFactory() {
        return xaqcf;
    }

    /**
     * Create a Queue and bind it to the registry
     */
    public Queue createQueue(String name) throws Exception {
        TraceJms.logger.log(BasicLevel.DEBUG,"");
        Queue queue = null;
        try {
            queue = (Queue) ictx.lookup(name);
        } catch (Exception e) {
            queue = admin.createQueue(0);
            ictx.rebind(name, queue);
        }
        admin.setWriter(user, queue);
        admin.setReader(user, queue);
        admin.setWriter(jonasUser, queue);
        admin.setReader(jonasUser, queue);
        return queue; 
    }

    /**
     * Create a Topic and bind it to the registry
     */
    public Topic createTopic(String name) throws Exception {
        TraceJms.logger.log(BasicLevel.DEBUG,"");
        Topic topic = null;
        try {
            topic = (Topic) ictx.lookup(name);
        } catch (Exception e) {
            topic = admin.createTopic(0);
            ictx.rebind(name, topic);
        }
        admin.setWriter(user, topic);
        admin.setReader(user, topic);
        admin.setWriter(jonasUser, topic);
        admin.setReader(jonasUser, topic);
        return topic;
    }
}
