/*
 * @(#) InternalTransactionContext.java 
 *
 * JOTM: Java Open Transaction Manager 
 *
 * This project is developed inside the ObjectWeb Consortium,
 * http://www.objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Contributor(s): ______________________________________.
 *
 */
package org.objectweb.jonas_tm;

// java import
import java.io.Serializable;

// javax import (JTA)
import javax.transaction.xa.Xid;

/**
 * Classe <code>InternalTransactionContext</code> is a generic implmentation of the JOTM
 * Transaction Context. This Context is used by JOTM and by the Current Object. It can't
 * be propagate, it have to be transalate for each Transport Layer Transaction Prapagation 
 * implmentation 
 * 
 * @author  Guillaume Riviere (Guillaume.Riviere@inrialpes.fr)
 * @version 1.0, 12/09/2002
 */
public class InternalTransactionContext implements TransactionContext {

    /**
     * @serial
     */
    private int timeout;

    /**
     * @serial
     */
    private Coordinator coordinator;

    /**
     * @serial
     */
    private Xid xid;

    /**
     * Build a new TransactionContext (from JTA layer)
     */
    public InternalTransactionContext(int t, Coordinator c, Xid x) {
	timeout = t;
	coordinator = c;
	xid = x;
    }

    /**
     * Get the timeout associated with the transaction
     */
    public int getTimeout() {
	return timeout;
    }

    /**
     * Get the coordinator associated with the transaction
     */
    public Coordinator getCoordinator() {
	return coordinator;
    }

    /**
     * Set the coordinator associated with the transaction
     */
    public void setCoordinator(Coordinator coord) {
	this.coordinator = coord;
    }

    /**
     * Get the Terminator associated with the transaction
     */
    public Terminator getTerminator() {
	return (Terminator) coordinator;
    }

    /**
     * Get the control associated with the transaction
     */
    public Control getControl() {
	return (Control) coordinator;
    }

    /**
     * Get the Xid associated with the transaction
     */
    public Xid getXid() {
	return xid;
    }

}
