/*
 * @(#) JTAClientTransactionInterceptor
 *
 * Copyright (C) 2002 - INRIA (www.inria.fr)
 * 
 * JOTM: Java Open Transaction Manager
 *
 * This library is developed inside the ObjectWeb Consortium,
 * http://www.objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * 
 *
 */
package org.objectweb.jonas_tm.jta.rmi;

// java import
import java.io.IOException;

// Carol JRMP API Import
import org.objectweb.carol.rmi.jrmp.interceptor.JClientRequestInterceptor;
import org.objectweb.carol.rmi.jrmp.interceptor.JClientRequestInfo;

// JOTM import
import org.objectweb.jonas_tm.Current;
import org.objectweb.jonas_tm.TransactionContext;

/**
 * Class <code>JTAClientTransactionInterceptor</code> is a JRMP Transaction client interceptor for 
 * Transaction Context propagation
 *
 * @author  Guillaume Riviere (Guillaume.Riviere@inrialpes.fr)
 * @version 1.0, 15/07/2002
 */
public class JTAClientTransactionInterceptor implements JClientRequestInterceptor {

    /**
     * transaction context id
     */
    public static int TX_CTX_ID = 0;

    /**
     * current object
     */
    private static Current current = null;

    /**
     * interceptor name
     */
    private static String interceptorName = "JTAClientTransactionInterceptor";

    /**
     * constructor 
     */
    public JTAClientTransactionInterceptor() {
    }

    /**
     * get the name of this interceptor
     * @return name
     */
    public String name() {
	return interceptorName;
    } 

    /**
     * send client context with the request. The sendingRequest method of the JPortableInterceptors
     * is called prior to marshalling arguments and contexts
     * @param JClientRequestInfo jri the jrmp client info 
     * @exception IOException if an exception occur with the ObjectOutput
     */
    public void send_request(JClientRequestInfo jri) throws IOException {
	try {
	    if (current == null) current = Current.getCurrent();
	    if (current != null) {
		// get the Transaction Context (null if there is no transaction)
		TransactionContext txCtx = current.getPropagationContext(true);
		if (txCtx != null) {
		    JTATransactionServiceContext jtasc = new JTATransactionServiceContext();
		    jtasc.setContext(txCtx, false);
		    jri.add_request_service_context(jtasc);
		}
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Receive reply interception
     * @param JClientRequestInfo jri the jrmp client info 
     * @exception IOException if an exception occur with the ObjectOutput
     */
    public void receive_reply(JClientRequestInfo jri)throws IOException{
	if (current == null) current = Current.getCurrent();
	if (current != null) {
	    JTATransactionServiceContext jtasc = (JTATransactionServiceContext)jri.get_reply_service_context(TX_CTX_ID);
	    if (jtasc != null) {
		// put into the the Current object (true for client side context
		current.setPropagationContext(jtasc.getTransactionContext(), true);
	    }	
	}
    }

    // empty method
    public void send_poll(JClientRequestInfo jri) throws IOException {
    }

    public void receive_exception(JClientRequestInfo jri) throws IOException {
// 	if (current == null) current = Current.getCurrent();
// 	if (current != null) {
// 	    JTATransactionServiceContext jtasc = (JTATransactionServiceContext)jri.get_reply_service_context(TX_CTX_ID);
// 	    if (jtasc != null) {
// 		// put into the the Current object (true for client side context
// 		current.setPropagationContext(jtasc.getTransactionContext(), true);
// 	    }	
// 	}
    }

    public void receive_other(JClientRequestInfo jri) throws IOException {
	if (current == null) current = Current.getCurrent();
	if (current != null) {
	    JTATransactionServiceContext jtasc = (JTATransactionServiceContext)jri.get_reply_service_context(TX_CTX_ID);
	    if (jtasc != null) {
		// put into the the Current object (true for client side context
		current.setPropagationContext(jtasc.getTransactionContext(), true);
	    }	
	}
    }
}

