/*
 * @(#) JTAServerTransactionInterceptor
 *
 * Copyright (C) 2002 - INRIA (www.inria.fr)
 * 
 * JOTM: Java Open Transaction Manager
 *
 * This library is developed inside the ObjectWeb Consortium,
 * http://www.objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * 
 *
 */
package org.objectweb.jonas_tm.jta.rmi;

//XXX to remove, only for tests
import org.objectweb.carol.util.perfs.*;

// java import
import java.io.IOException;

//carol JRMP Interceptor API Import
import org.objectweb.carol.rmi.jrmp.interceptor.JServerRequestInterceptor;
import org.objectweb.carol.rmi.jrmp.interceptor.JServerRequestInfo;

// JOTM import
import org.objectweb.jonas_tm.Current;
import org.objectweb.jonas_tm.TransactionContext;

/**
 * Class <code>JTAServerTransactionInterceptor</code> is a JRMP Transaction server interceptor for 
 * Transaction Context propagation
 * 
 * @author  Guillaume Riviere (Guillaume.Riviere@inrialpes.fr)
 * @version 1.0, 13/09/2002
 */
public class JTAServerTransactionInterceptor implements  JServerRequestInterceptor {

    /**
     * transaction context id
     */
    public static int TX_CTX_ID = 0;

    /**
     * current object
     */
    private static Current current = null;

    /**
     * interceptor name
     */
    private static String interceptorName = "JTAServerTransactionInterceptor";   

    /**
     * constructor 
     */
    public JTAServerTransactionInterceptor() {
    }

    /**
     * Receive request 
     * @param JServerRequestInfo the jrmp server request information
     * @exception IOException if an exception occur with the ObjectOutput
     */
    public void receive_request(JServerRequestInfo jri) throws IOException {
	if (current == null) current = Current.getCurrent();
	if (current != null) {
	    JTATransactionServiceContext jtasc = (JTATransactionServiceContext)jri.get_request_service_context(TX_CTX_ID);
	    if (jtasc != null) {
		// put into the the Current object (true for client side context
		current.setPropagationContext(jtasc.getTransactionContext(), false);
	    }
	}
    }

    /**
     * send reply with context
     * @param JServerRequestInfo the jrmp server request information
     * @exception IOException if an exception occur with the ObjectOutput
     */
    public void send_reply(JServerRequestInfo jri)throws IOException {
	if (current == null) current = Current.getCurrent();
	if (current != null) {
	    // get the Transaction Context (null if there is no transaction)
	    TransactionContext txCtx = current.getPropagationContext(false);
	    if (txCtx != null) {		
		JTATransactionServiceContext jtasc = new JTATransactionServiceContext();
		jtasc.setContext(txCtx,true);
		jri.add_reply_service_context(jtasc);
		current.setPropagationContext(null, false);
	    }
	}
    }

    
     /**
     * get the name of this interceptor
     * @return name
     */
    public String name() {
	return interceptorName;
    } 


    public void send_exception(JServerRequestInfo jri) throws IOException {
	if (current == null) current = Current.getCurrent();
	if (current != null) {
	    // get the Transaction Context (null if there is no transaction)
	    TransactionContext txCtx = current.getPropagationContext(false);
	    if (txCtx != null) {		
		JTATransactionServiceContext jtasc = new JTATransactionServiceContext();
		jtasc.setContext(txCtx,true);
		jri.add_reply_service_context(jtasc);
		current.setPropagationContext(null, false);
	    }
	}
    }

    public void send_other(JServerRequestInfo jri) throws IOException {
	if (current == null) current = Current.getCurrent();
	if (current != null) {
	    // get the Transaction Context (null if there is no transaction)
	    TransactionContext txCtx = current.getPropagationContext(false);
	    if (txCtx != null) {		
		JTATransactionServiceContext jtasc = new JTATransactionServiceContext();
		jtasc.setContext(txCtx,true);
		jri.add_reply_service_context(jtasc);
		current.setPropagationContext(null, false);
	    }
	}
    }
}
