/*
 * @(#) JTATransactionServiceContext
 *
 * Copyright (C) 2002 - INRIA (www.inria.fr)
 *
 * JOTM: Java Open Transaction Manager
 *
 * This library is developed inside the ObjectWeb Consortium,
 * http://www.objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * 
 *
 */
package org.objectweb.jonas_tm.jta.rmi;

// java import 
import java.io.IOException;
import java.io.Externalizable;
import java.io.ObjectOutput;
import java.io.ObjectInput;
import java.io.ObjectStreamException;

// carol import
import org.objectweb.carol.rmi.jrmp.interceptor.JServiceContext;

// JOTM import
import org.objectweb.jonas_tm.InternalTransactionContext;
import org.objectweb.jonas_tm.TransactionContext;
import org.objectweb.jonas_tm.Coordinator;
import org.objectweb.jonas_tm.XidImpl;
import javax.transaction.xa.Xid;

/**
 * Class <code>JTATransactionServiceContext</code> is a JRMP Class for Transaction
 * Context Propagation
 * 
 * @author  Guillaume Riviere (Guillaume.Riviere@inrialpes.fr)
 * @version 1.0, 13/09/2002
 */
public class JTATransactionServiceContext implements JServiceContext, Externalizable {

    /**
     * Transaction Context
     */
    transient TransactionContext txCtx = null;  
      
    /**
     * true is this is a reply
     */
    transient boolean isReply;

    /**
     * context id
     */
    private transient int context_id;

    /**
     * empty constructor for extarenalizable
     */
    public JTATransactionServiceContext() {
	this.context_id=JTAClientTransactionInterceptor.TX_CTX_ID;
    }

    /**
     * the JServiceContext id 
     */ 
    public int getContextId() {
	return context_id;
    }

    /**
     * constructor
     * @param int the context_i
     * @param TransactionContext the RMI (Serializable) Transaction Context 
     */
    public void setContext(TransactionContext txCtx, boolean isReply) {
	this.txCtx=txCtx;
	this.isReply=isReply;
    }

    /**
     * get the transaction context
     * @return TransactionContext the Transaction context
     */
    public TransactionContext getTransactionContext() {
	return txCtx;
    }  

    /**
     * readExternal to initialise Transaction context
     * @param in the object input 
     */
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
	// read Xid
	int fid = in.readInt();
	byte [] gti = new byte [in.readInt()];
	in.read(gti);
	byte [] bq = new byte [in.readInt()];
	in.read(bq);	
	Xid xid = new XidImpl(fid, gti, bq);
	// read Coordinator
	Coordinator coor = (Coordinator)in.readObject();
	// read timeout
	int timeout = in.readInt();
	this.txCtx=new InternalTransactionContext(timeout, coor, xid);
    }

    /**
     * writeExternal to send Transaction context
     * @param out the object output
     */
    public void writeExternal(ObjectOutput out) throws IOException {
	// send Xid
	Xid xid = txCtx.getXid();
	out.writeInt(xid.getFormatId());
	out.writeInt(xid.getGlobalTransactionId().length);   
	out.write(xid.getGlobalTransactionId());
	out.writeInt(xid.getBranchQualifier().length);   
	out.write(xid.getBranchQualifier());
	// send Coordinator
	out.writeObject(txCtx.getCoordinator());
	// send timeout
	out.writeInt(txCtx.getTimeout());
    }
}
