/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Jeff Mesnil.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */



package org.objectweb.security.context;

import java.io.Serializable;

import java.security.Principal;
import java.util.ArrayList;
import java.util.Arrays;
/**
 * Implementation of the JOnAS Security Context
 * @author Jeff Mesnil
 */

public class SecurityContext implements Serializable {

    /**
     * Name of the principal
     */
    private String principalName = null ;

    /**
     * List of roles of the principal
     */
    private final String[] roles;


    /**
     * Constructor SecurityContext
     * use the default principal name
     *
     */
    public SecurityContext () {
        this.principalName = "JOnAS_client";
        this.roles = null;
    }

    /**
     * Constructor SecurityContext
     * @param principalName principal name
     * @param roles the roles of the principal
     */
    public SecurityContext (String principalName, String[] roles) {
        this.principalName = principalName;
        this.roles = roles;
    }

    /**
     * Constructor SecurityContext
     * @param principalName principal name
     * @param roles the list of the roles of the principal
     */
    public SecurityContext (String principalName, ArrayList arrayRoles) {
        this.principalName = principalName;
        //Convert list into array
        String[] r = new String[arrayRoles.size()];
        r = (String[]) arrayRoles.toArray(r);
        this.roles = r;
    }


    /**
     * Constructor SecurityContext
     * @param principalName principal name
     *
     */
    public SecurityContext (String principalName) {
        this.principalName = principalName;
        this.roles = null;
    }


    /**
     * Method getCallerPrincipal
     * @return the Principal in the Security Context
     *
     */
    public Principal getCallerPrincipal() {
        return new Principal () {
                public String getName() {
                    return principalName;
                }
                
                public boolean equals (Object o) {
                    if (o instanceof Principal) {
                        return principalName.equals(((Principal) o).getName());
                    }
                    return false;
                }

                public String toString () {
                    return "name = " + principalName;
                }
            };
    }

    /**
     * Return the roles of the principal
     * @return roles
     */
    public String[] getCallerPrincipalRoles() {
        return roles;
    }

    
    /**
     * Method toString
     * @return String a string representation of the object
     */
    public String toString () {
        String txt =  "principal : name = " + principalName + "\n";
        if (roles != null) {
            for(int i = 0; i < roles.length; i++) {
                txt += "role[" + i + "] = " + roles[i];
            }
        }
        return txt;
    }
 
}

