package org.objectweb.ws.addressbook_client;

import org.objectweb.ws.addressbook_client.gen.Address;
import org.objectweb.ws.addressbook_client.gen.AddressBook;
import org.objectweb.ws.addressbook_client.gen.AddressBookService;
import org.objectweb.ws.addressbook_client.gen.AddressBookServiceLocator;

public class AddressBookClient {

    private AddressBookService abServ = null;
    private AddressBook        ab     = null;

    public static void main(String [] args) {
        AddressBookClient client = new AddressBookClient();

        try {
            // getAddress
            Address a1 = client.getAddress("Xavier Delplanque");
            System.out.println("-- Address Returned : --");
            System.out.println(" - name    : "+a1.getName());
            System.out.println(" - city    : "+a1.getCity());
            System.out.println(" - zip     : "+a1.getZip());
            System.out.println(" - country : "+a1.getCountry());
            System.out.println(" - phone   : "+a1.getPhone());

            System.out.println("");

            // getAllAddresses
            Address[] adds = client.getAllAddresses();
            System.out.println("-- Addresses Returned : --");
            for (int i = 0 ; i < adds.length ; i++) {
                Address a = adds[i];
                System.out.println(" - name    : "+a.getName());
                System.out.println(" - city    : "+a.getCity());
                System.out.println(" - zip     : "+a.getZip());
                System.out.println(" - country : "+a.getCountry());
                System.out.println(" - phone   : "+a.getPhone());
                System.out.println("---------------------------");
            }

            System.out.println("");

            // countAddresses
            int count = client.countAddresses();
            System.out.println("-- Number of Addresses : " + count);

            System.out.println("");

            // getOwner
            String owner = client.getOwner();
            System.out.println("-- AddressBook Owner : " + owner);

            System.out.println("");

            // addEntry
            Address a2 = new Address();
            a2.setName("JOnAS");
            a2.setCity("Echirolles");
            a2.setZip(38100);
            a2.setPhone("0476291526");
            String res = client.addEntry(a2);
            System.out.println("-- addEntry return : " + res);
            
        }
        catch(java.rmi.RemoteException re) {
            re.printStackTrace(System.err);
        }

    }

    public AddressBookClient() {

        abServ = new AddressBookServiceLocator();

        try {
            ab = abServ.getAddressBookService();
        }
        catch(javax.xml.rpc.ServiceException se) {
            se.printStackTrace(System.err);
        }
    }

    public Address getAddress(String name) throws java.rmi.RemoteException {
        return ab.getAddress(name);
    }

    public Address[] getAllAddresses() throws java.rmi.RemoteException {
        return ab.getAllAddresses();
    }

    public int countAddresses() throws java.rmi.RemoteException {
        return ab.countAddresses();
    }

    public String getOwner() throws java.rmi.RemoteException {
        return ab.getOwner();
    }

    public String addEntry(Address a) throws java.rmi.RemoteException {
        return ab.addEntry(a);
    }


}
