package org.objectweb.ws.addressbook_client;

/* The DII CLient just have to know about the serializables Objects */
import org.objectweb.ws.addressbook_client.gen.Address;

/* Axis Libraries needed for a DII Call */
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;

/* Ser / Deser */
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;
import org.apache.axis.encoding.ser.ArraySerializerFactory;
import org.apache.axis.encoding.ser.ArrayDeserializerFactory;

import javax.xml.namespace.QName;
import javax.xml.rpc.ParameterMode;

public class AddressBookClientDII {
    
    private Service        service     = null;
    private java.net.URL   endpointURL = null;
    
    public static void main(String [] args) {
        AddressBookClientDII client = new AddressBookClientDII("http://localhost:"+args[0]+"/addressbook/services/AddressBookService");

        try {
            // getAddress
            Address a1 = client.getAddress("Xavier Delplanque");
            System.out.println("-- Address Returned : --");
            System.out.println(" - name    : "+a1.getName());
            System.out.println(" - city    : "+a1.getCity());
            System.out.println(" - zip     : "+a1.getZip());
            System.out.println(" - country : "+a1.getCountry());
            System.out.println(" - phone   : "+a1.getPhone());

            System.out.println("");

            // getAllAddresses
            Address[] adds = client.getAllAddresses();
            System.out.println("-- Addresses Returned : --");
            for (int i = 0 ; i < adds.length ; i++) {
                Address a = adds[i];
                System.out.println(" - name    : "+a.getName());
                System.out.println(" - city    : "+a.getCity());
                System.out.println(" - zip     : "+a.getZip());
                System.out.println(" - country : "+a.getCountry());
                System.out.println(" - phone   : "+a.getPhone());
                System.out.println("---------------------------");
            }

            System.out.println("");

            // countAddresses
            int count = client.countAddresses();
            System.out.println("-- Number of Addresses : " + count);

            System.out.println("");

            // getOwner
            String owner = client.getOwner();
            System.out.println("-- AddressBook Owner : " + owner);

            System.out.println("");

            // addEntry
            Address a2 = new Address();
            a2.setName("JOnAS");
            a2.setCity("Echirolles");
            a2.setZip(38100);
            a2.setPhone("0476291526");
            String res = client.addEntry(a2);
            System.out.println("-- addEntry return : " + res);
            
        }
        catch(java.rmi.RemoteException re) {
            re.printStackTrace(System.err);
        }
        catch(javax.xml.rpc.ServiceException se) {
            se.printStackTrace(System.err);
        }

    }

    public AddressBookClientDII(String ep) {

        service = new Service();
        try {
            endpointURL = new java.net.URL(ep);
        }
        catch (java.net.MalformedURLException mue) {}

    }

    public Call customCall(String methodName) throws javax.xml.rpc.ServiceException{
        Call call = (Call) service.createCall();
        call.setTargetEndpointAddress( endpointURL );
        call.setOperationName( new QName("AddressBookService", methodName) );
        return call;
    }

    public Address getAddress(String name) throws java.rmi.RemoteException, javax.xml.rpc.ServiceException {
        Call call = customCall("getAddress");
        QName addressQName = new QName("http://addressbook.ws.tests.objectweb.org", "Address");
        call.registerTypeMapping(Address.class,
                                 addressQName,
                                 new BeanSerializerFactory(Address.class, addressQName) ,
                                 new BeanDeserializerFactory(Address.class, addressQName));
        return (Address) call.invoke(new Object[]{name});
    }

    public Address[] getAllAddresses() throws java.rmi.RemoteException, javax.xml.rpc.ServiceException {
        Call call = customCall("getAllAddresses");
        QName addressQName = new QName("http://addressbook.ws.tests.objectweb.org", "Address");
        call.registerTypeMapping(Address.class,
                                 addressQName,
                                 new BeanSerializerFactory(Address.class, addressQName) ,
                                 new BeanDeserializerFactory(Address.class, addressQName));
        QName addressArrayQName = new QName("http://addressbook.ws.tests.objectweb.org", "ArrayOfAddress");
        call.registerTypeMapping(Address[].class,
                                 addressArrayQName,
                                 new ArraySerializerFactory() ,
                                 new ArrayDeserializerFactory());

        return (Address[]) call.invoke(new Object[]{});
    }

    public int countAddresses() throws java.rmi.RemoteException, javax.xml.rpc.ServiceException {
        Call call = customCall("countAddresses");
        call.setReturnType( org.apache.axis.encoding.XMLType.XSD_INT );
        return ((Integer) call.invoke(new Object[]{})).intValue();
    }

    public String getOwner() throws java.rmi.RemoteException, javax.xml.rpc.ServiceException {
        Call call = customCall("getOwner");
        call.setReturnType( org.apache.axis.encoding.XMLType.XSD_STRING );
        return (String) call.invoke(new Object[]{});
    }

    public String addEntry(Address a) throws java.rmi.RemoteException, javax.xml.rpc.ServiceException {
        Call call = customCall("addEntry");
        QName addressQName = new QName("http://addressbook.ws.tests.objectweb.org", "Address");
        call.registerTypeMapping(Address.class,
                                 addressQName,
                                 new BeanSerializerFactory(Address.class, addressQName) ,
                                 new BeanDeserializerFactory(Address.class, addressQName));
        return (String) call.invoke(new Object[]{a});
    }


}
