// AddressBookClientDIISLR.java
// Stateless Session bean

package org.objectweb.ws.addressbook_client_ejb.beans.ab_diiclient;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.RemoveException;
import javax.ejb.EJBObject;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import org.objectweb.ws.addressbook_client_ejb.gen.Address;

/* Axis Libraries needed for a DII Call */
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;

/* Ser / Deser */
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;
import org.apache.axis.encoding.ser.ArraySerializerFactory;
import org.apache.axis.encoding.ser.ArrayDeserializerFactory;

import javax.xml.namespace.QName;
import javax.xml.rpc.ParameterMode;

/**
 *
 */
public class AddressBookClientDIISLR implements SessionBean {

    static private Logger logger = null;
    SessionContext ejbContext;

    // ------------------------------------------------------------------
    // SessionBean implementation
    // ------------------------------------------------------------------


    public void setSessionContext(SessionContext ctx) {
        if (logger == null) {
            logger = Log.getLogger("org.objectweb.jonas_tests");
        }
        logger.log(BasicLevel.DEBUG, "");
        ejbContext = ctx;
    }
        

    public void ejbRemove() {
        logger.log(BasicLevel.DEBUG, "");
    }
        

    public void ejbCreate() throws CreateException {
        logger.log(BasicLevel.DEBUG, "Service instance creation");
        service = new Service();
    }

    public void ejbPassivate() {
        logger.log(BasicLevel.DEBUG, "");
    }

    public void ejbActivate() {
        logger.log(BasicLevel.DEBUG, "");
    }
    
    // ------------------------------------------------------------------
    // AddressBookClientDII implementation
    // ------------------------------------------------------------------

    private Service        service     = null;
    private java.net.URL   endpointURL = null;

    public void setEndpointURL(java.net.URL url) {
        endpointURL = url;
    }

    public Call customCall(String methodName) throws javax.xml.rpc.ServiceException{
        Call call = (Call) service.createCall();
        call.setTargetEndpointAddress( endpointURL );
        call.setOperationName( new QName("AddressBookService", methodName) );
        return call;
    }

    public Address getAddress(String name) throws java.rmi.RemoteException {
        Call call = null;
        try {
            call = customCall("getAddress");
        }
        catch(javax.xml.rpc.ServiceException se) {
            throw new java.rmi.RemoteException(se.getMessage());
        }
        QName addressQName = new QName("http://addressbook.ws.tests.objectweb.org", "Address");
        call.registerTypeMapping(Address.class,
                                 addressQName,
                                 new BeanSerializerFactory(Address.class, addressQName) ,
                                 new BeanDeserializerFactory(Address.class, addressQName));
        return (Address) call.invoke(new Object[]{name});
    }

    public Address[] getAllAddresses() throws java.rmi.RemoteException {
        Call call = null;
        try {
            call = customCall("getAllAddresses");
        }
        catch(javax.xml.rpc.ServiceException se) {
            throw new java.rmi.RemoteException(se.getMessage());
        }
        QName addressQName = new QName("http://addressbook.ws.tests.objectweb.org", "Address");
        call.registerTypeMapping(Address.class,
                                 addressQName,
                                 new BeanSerializerFactory(Address.class, addressQName) ,
                                 new BeanDeserializerFactory(Address.class, addressQName));
        QName addressArrayQName = new QName("http://addressbook.ws.tests.objectweb.org", "ArrayOfAddress");
        call.registerTypeMapping(Address[].class,
                                 addressArrayQName,
                                 new ArraySerializerFactory() ,
                                 new ArrayDeserializerFactory());

        return (Address[]) call.invoke(new Object[]{});
    }

    public int countAddresses() throws java.rmi.RemoteException {
        Call call = null;
        try {
            call = customCall("countAddresses");
        }
        catch(javax.xml.rpc.ServiceException se) {
            throw new java.rmi.RemoteException(se.getMessage());
        }
        call.setReturnType( org.apache.axis.encoding.XMLType.XSD_INT );
        return ((Integer) call.invoke(new Object[]{})).intValue();
    }

    public String getOwner() throws java.rmi.RemoteException {
        Call call = null;
        try {
            call = customCall("getOwner");
        }
        catch(javax.xml.rpc.ServiceException se) {
            throw new java.rmi.RemoteException(se.getMessage());
        }
        call.setReturnType( org.apache.axis.encoding.XMLType.XSD_STRING );
        return (String) call.invoke(new Object[]{});
    }

    public String addEntry(Address a) throws java.rmi.RemoteException {
        Call call = null;
        try {
            call = customCall("addEntry");
        }
        catch(javax.xml.rpc.ServiceException se) {
            throw new java.rmi.RemoteException(se.getMessage());
        }
        QName addressQName = new QName("http://addressbook.ws.tests.objectweb.org", "Address");
        call.registerTypeMapping(Address.class,
                                 addressQName,
                                 new BeanSerializerFactory(Address.class, addressQName) ,
                                 new BeanDeserializerFactory(Address.class, addressQName));
        return (String) call.invoke(new Object[]{a});
    }


    

}

