package org.objectweb.ws.addressbook_web.servlets.ab;

import java.util.HashMap;
import java.util.Collection;

public class AddressBookService implements AddressBook {

    private static String owner = "Guillaume Sauthier";
    private static HashMap addresses = new HashMap();

    static {
        Address a1 = new Address();
        a1.setName("Xavier Delplanque");
        a1.setPhone("045000000");
        a1.setCity("Grenoble");
        a1.setZip(38000);
        addresses.put(a1.getName(), a1);

        Address a2 = new Address();
        a2.setName("Simon Andriot");
        a2.setPhone("045000001");
        a2.setCity("Saint Martin d'Heres");
        a2.setZip(38000);
        addresses.put(a2.getName(), a2);
    }

    /**
     * Default Constructor
     */
    public AddressBookService() {

    }

    /**
     * Return the address associated with the name in param
     * @param name The name of the person we want the address
     * @return The address of this person
     */
    public Address getAddress(String name) {
        if ( addresses.containsKey(name) )
            return (Address) addresses.get(name);
        else
            return null;
    }

    /**
     * Return all the addresses in the address book
     * @return All the addresses in the address book
     */
    public Address[] getAllAddresses() {
        return (Address[]) addresses.values().toArray(new Address[addresses.size()]);
    }

    /**
     * Return the number of entries in the address book
     * @return The number of entries in the address book
     */
    public int countAddresses() {
        return addresses.size();
    }

    /**
     * Return the name of the Address book owner
     * @return The owner's name
     */
    public String getOwner() {
        return owner;
    }

    /**
     * Add an Address in the address book
     * @param a The address to add
     * @return "Failed" with error or "Success"
     */
    public String addEntry(Address a) {
        if ( addresses.containsKey(a.getName()) ) {
            return "Failed : Name already in Book!";
        }
        else {
            addresses.put(a.getName(), a);
            return "Success : "+a.getName() + " added!";
        }
    }
}
