/*
 * @(#) SecurityInterceptor.java	1.0 02/07/15
 *
 * Copyright (C) 2002 - INRIA (www.inria.fr)
 * 
 * JONAS
 *
 * This library is developed inside the ObjectWeb Consortium,
 * http://www.objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * 
 *
 */
package org.objectweb.jonas.security.interceptors.iiop;

import org.omg.CORBA.LocalObject;
import org.objectweb.security.context.SecurityContext;
import java.util.Collection;
import java.util.ArrayList;


public abstract class SecurityInterceptor extends LocalObject {

    /**
     * security context id
     */
    public static int SEC_CTX_ID = 101;

    /**
     * Custom UTF8 marshalling SecurityContext 
     * @return byte [] the marshalled context
     * @param SecurityContext
     */
    public byte [] marshallSecurityContext(SecurityContext ctx) {
	byte [] result = null;
	try {
	    if (ctx!=null) {
		// build a new byte from ctx
		ArrayList cResult = new ArrayList();
		// add the pname
		addBytes(cResult, ctx.getCallerPrincipal().getName());
		
		// add the roles
		String [] roles = ctx.getCallerPrincipalRoles(); 
		if (roles!=null) {
		    for (int i=0; i<roles.length;i++) {
			addBytes(cResult, roles[i]);
		    }
		} 
		
		// result casting
		result = new byte[cResult.size()];
		for (int i=0; i<cResult.size(); i++) {
		    result[i]=((Byte)cResult.get(i)).byteValue();
		}
	    }

	} catch (java.io.UnsupportedEncodingException uee) {
	    uee.printStackTrace();
	}
	return result;
    }

    /**
     * Custom UTF8 marshalling SecurityContext 
     * @param byte [] the marshalled context
     * @return SecurityContext
     */
    public SecurityContext unmarshallSecurityContext(byte [] byteCtx) {
	SecurityContext result = null;
	try {
	    if ((byteCtx!=null)&&(byteCtx.length>0)) {
		int counter=0;
		// get the pName
		int pSize=byteCtx[counter];
		counter++;
		byte [] pName=new byte [pSize];
		for (int j=0; j<pSize; j++) {
		    pName[j]=byteCtx[counter];
		    counter++;
		}
		String pNameString=new String(pName, "UTF8");
		
		// get the roles if existed 
		ArrayList roles= new ArrayList();
		boolean otherRoles=(byteCtx.length>counter);
		while (otherRoles) {
		    // get the next String and put it inside the ArrayList
		    int rSize=byteCtx[counter];
		    counter++;
		    byte [] rName=new byte [rSize];
		    for (int j=0; j<rSize; j++) {
			rName[j]=byteCtx[counter];
			counter++;
		    }
		    roles.add(new String(rName, "UTF8"));
		    otherRoles=(byteCtx.length>counter);
		}
		if (roles.size()>0) {
		    result = new SecurityContext(pNameString, roles);
		} else {
		    result = new SecurityContext(pNameString);
		}
	    }
	} catch (java.io.UnsupportedEncodingException uee) {
	    uee.printStackTrace();
	}
	return result;
    }


    private void addBytes(Collection c, String toAdd) throws java.io.UnsupportedEncodingException {
	byte [] b = toAdd.getBytes("UTF8");
	c.add(new Byte((byte)b.length));
	for (int i=0; i<b.length; i++) {
	    c.add(new Byte(b[i]));
	}
    }

    public String contextString(SecurityContext scx) {
	String result = "context principal name=<";
	result+=scx.getCallerPrincipal().getName();
	result+="> ";
	// add the roles
	String [] roles = scx.getCallerPrincipalRoles(); 
	if (roles!=null) {
	    result+="roles=[<";
	    for (int i=0; i<roles.length-1;i++) {
		result+=roles[i]+">, <";
	    }
	    result+=roles[roles.length-1]+">]";
	} 
	return result;
    }
    
}

