/*
 * @(#) OTSInterceptor.java	1.0 02/07/15
 *
 * Copyright (C) 2002 - INRIA (www.inria.fr)
 * 
 * JOTM: Java Open Transaction Manager
 *
 * This library is developed inside the ObjectWeb Consortium,
 * http://www.objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * 
 *
 */
package org.objectweb.jonas_tm.ots;

import java.io.*;
import java.util.*;
import java.lang.*;
import java.text.*;
import org.omg.CORBA.*;
import org.omg.Dynamic.*;
import org.omg.DynamicAny.*;
import org.omg.DynamicAny.DynAnyPackage.*;
import org.omg.DynamicAny.DynAnyFactoryPackage.*;
import org.omg.PortableInterceptor.*;
import org.omg.PortableInterceptor.ORBInitInfoPackage.*;
import org.omg.IOP.Codec;
import org.omg.CosTransactions.PropagationContextHelper;



public abstract class OTSInterceptor extends LocalObject {

    // Used to encode/decode the call
    protected Codec codec;


    // for identifying client or server.
    protected DynAnyFactory dynAnyFactoryS_;


    
    // Service context identifier
    protected final int TX_CTX_ID = 100;


    /**
     * constructor
     */

    public OTSInterceptor(ORBInitInfo info) {

	
	// Get the codec factory
	org.omg.IOP.CodecFactory factory = info.codec_factory();
	if(factory == null) throw new RuntimeException();

	// Create codec
	org.omg.IOP.Encoding how = new org.omg.IOP.Encoding();
	how.major_version = 1;
	how.minor_version = 0;
	how.format = org.omg.IOP.ENCODING_CDR_ENCAPS.value;

	try {
	    codec = factory.create_codec(how);
	} catch(org.omg.IOP.CodecFactoryPackage.UnknownEncoding ex) {
	    throw new RuntimeException();
	}

	if(codec == null) throw new RuntimeException();

	//
	// Get the dynamic any factory
	//
	DynAnyFactory dynAnyFactory = null;
	try {
	    org.omg.CORBA.Object obj = info.resolve_initial_references("DynAnyFactory");
	    dynAnyFactory = DynAnyFactoryHelper.narrow(obj);
	} catch(InvalidName ex) {
	    throw new RuntimeException();
	} catch(org.omg.CORBA.BAD_PARAM ex) {
	    throw new RuntimeException();
	}

	dynAnyFactoryS_ = dynAnyFactory;
    } // end constructor



    protected Any create_any() 
	throws org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode{
	org.omg.DynamicAny.DynAny dynAny = dynAnyFactoryS_.create_dyn_any_from_type_code(PropagationContextHelper.type());
	return dynAny.to_any();
		    
    }

}

